# flake8: noqa

from souschef.meal.models import (
    Component,
    Component_ingredient,
    Incompatibility,
    Ingredient,
    Restricted_item,
)


def print_rows(q, heading=""):
    pass
    print(
        "\n-----------------------------------------------------\n",
        # q, "\n",
        heading,
    )
    for row in q.all():
        print(row)


def insert_all():
    print("\nSTART dataload")

    # Rename existing ingredients, components, restricted_items

    items = Ingredient.objects.all()
    for item in items:
        item.name = "OLD" + item.name
        item.save()
        print("Ingredient item name", item.name)

    items = Component.objects.all()
    for item in items:
        item.name = "OLD" + item.name
        item.save()
        print("Component item name", item.name)

    items = Restricted_item.objects.all()
    for item in items:
        item.name = "OLD" + item.name
        item.save()
        print("Restricted_item item name", item.name)

    # -----------------------------------------------------------------------------
    ing_1 = Ingredient.objects.create(name="Butter", ingredient_group="Dairy")
    ing_2 = Ingredient.objects.create(
        name="Cheese : Blue cheese", ingredient_group="Dairy"
    )
    ing_3 = Ingredient.objects.create(
        name="Cheese : Bocconcini", ingredient_group="Dairy"
    )
    ing_4 = Ingredient.objects.create(name="Cheese : Cheddar", ingredient_group="Dairy")
    ing_5 = Ingredient.objects.create(
        name="Cheese : Cottage cheese", ingredient_group="Dairy"
    )
    ing_6 = Ingredient.objects.create(name="Cheese : Feta", ingredient_group="Dairy")
    ing_7 = Ingredient.objects.create(
        name="Cheese : Goat cheese", ingredient_group="Dairy"
    )
    ing_8 = Ingredient.objects.create(
        name="Cheese : Mozzarella", ingredient_group="Dairy"
    )
    ing_9 = Ingredient.objects.create(
        name="Cheese : Parmesan", ingredient_group="Dairy"
    )
    ing_10 = Ingredient.objects.create(
        name="Cheese : Ricotta", ingredient_group="Dairy"
    )
    ing_11 = Ingredient.objects.create(name="Eggs", ingredient_group="Dairy")
    ing_12 = Ingredient.objects.create(name="Margarine", ingredient_group="Dairy")
    ing_13 = Ingredient.objects.create(name="Milk", ingredient_group="Dairy")
    ing_14 = Ingredient.objects.create(name="Tofu", ingredient_group="Dairy")
    ing_15 = Ingredient.objects.create(name="Yogurt", ingredient_group="Dairy")
    ing_16 = Ingredient.objects.create(
        name="Almond Powder ", ingredient_group="Dry and Canned goods"
    )
    ing_17 = Ingredient.objects.create(
        name="Almonds", ingredient_group="Dry and Canned goods"
    )
    ing_18 = Ingredient.objects.create(
        name="Capers", ingredient_group="Dry and Canned goods"
    )
    ing_19 = Ingredient.objects.create(
        name="Corn meal", ingredient_group="Dry and Canned goods"
    )
    ing_20 = Ingredient.objects.create(
        name="corn starch", ingredient_group="Dry and Canned goods"
    )
    ing_21 = Ingredient.objects.create(
        name="Creamed corn", ingredient_group="Dry and Canned goods"
    )
    ing_22 = Ingredient.objects.create(
        name="Egg noodles", ingredient_group="Dry and Canned goods"
    )
    ing_23 = Ingredient.objects.create(
        name="Linguini", ingredient_group="Dry and Canned goods"
    )
    ing_24 = Ingredient.objects.create(
        name="peanut butter", ingredient_group="Dry and Canned goods"
    )
    ing_25 = Ingredient.objects.create(
        name="peanuts ", ingredient_group="Dry and Canned goods"
    )
    ing_26 = Ingredient.objects.create(
        name="Pickles", ingredient_group="Dry and Canned goods"
    )
    ing_27 = Ingredient.objects.create(
        name="Polenta", ingredient_group="Dry and Canned goods"
    )
    ing_28 = Ingredient.objects.create(
        name="Rice noodles", ingredient_group="Dry and Canned goods"
    )
    ing_29 = Ingredient.objects.create(
        name="sesame seeds", ingredient_group="Dry and Canned goods"
    )
    ing_30 = Ingredient.objects.create(
        name="Sunflower seeds", ingredient_group="Dry and Canned goods"
    )
    ing_31 = Ingredient.objects.create(
        name="Tomato paste", ingredient_group="Dry and Canned goods"
    )
    ing_32 = Ingredient.objects.create(
        name="Walnuts", ingredient_group="Dry and Canned goods"
    )
    ing_33 = Ingredient.objects.create(
        name="Whole wheat lasagna", ingredient_group="Dry and Canned goods"
    )
    ing_34 = Ingredient.objects.create(
        name="Whole wheat linguini ", ingredient_group="Dry and Canned goods"
    )
    ing_35 = Ingredient.objects.create(
        name="Whole wheat macaroni", ingredient_group="Dry and Canned goods"
    )
    ing_36 = Ingredient.objects.create(
        name="Whole wheat penne ", ingredient_group="Dry and Canned goods"
    )
    ing_37 = Ingredient.objects.create(
        name="Whole wheat spaghetti", ingredient_group="Dry and Canned goods"
    )
    ing_38 = Ingredient.objects.create(name="Canned salmon", ingredient_group="Fish")
    ing_39 = Ingredient.objects.create(name="Canned tuna", ingredient_group="Fish")
    ing_40 = Ingredient.objects.create(name="Cod", ingredient_group="Fish")
    ing_41 = Ingredient.objects.create(name="Haddock", ingredient_group="Fish")
    ing_42 = Ingredient.objects.create(name="Pollock", ingredient_group="Fish")
    ing_43 = Ingredient.objects.create(name="Basil", ingredient_group="Fresh herbs")
    ing_44 = Ingredient.objects.create(name="Chives", ingredient_group="Fresh herbs")
    ing_45 = Ingredient.objects.create(name="Cilantro", ingredient_group="Fresh herbs")
    ing_46 = Ingredient.objects.create(name="Dill", ingredient_group="Fresh herbs")
    ing_47 = Ingredient.objects.create(name="Marjoram", ingredient_group="Fresh herbs")
    ing_48 = Ingredient.objects.create(name="Oregano", ingredient_group="Fresh herbs")
    ing_49 = Ingredient.objects.create(name="Parsley", ingredient_group="Fresh herbs")
    ing_50 = Ingredient.objects.create(name="Rosemary", ingredient_group="Fresh herbs")
    ing_51 = Ingredient.objects.create(name="Thyme", ingredient_group="Fresh herbs")
    ing_52 = Ingredient.objects.create(
        name="Barley (gluten)", ingredient_group="Grains"
    )
    ing_53 = Ingredient.objects.create(name="Basmati rice", ingredient_group="Grains")
    ing_54 = Ingredient.objects.create(name="Breadcrumbs", ingredient_group="Grains")
    ing_55 = Ingredient.objects.create(name="Brown rice", ingredient_group="Grains")
    ing_56 = Ingredient.objects.create(name="Buckwheat", ingredient_group="Grains")
    ing_57 = Ingredient.objects.create(
        name="Couscous (gluten)", ingredient_group="Grains"
    )
    ing_58 = Ingredient.objects.create(name="Millet", ingredient_group="Grains")
    ing_59 = Ingredient.objects.create(name="Oats", ingredient_group="Grains")
    ing_60 = Ingredient.objects.create(name="Quinoa", ingredient_group="Grains")
    ing_61 = Ingredient.objects.create(name="White flour", ingredient_group="Grains")
    ing_62 = Ingredient.objects.create(
        name="Whole wheat flour", ingredient_group="Grains"
    )
    ing_63 = Ingredient.objects.create(name="Wild rice", ingredient_group="Grains")
    ing_64 = Ingredient.objects.create(
        name="Black beans", ingredient_group="Legumineuse"
    )
    ing_65 = Ingredient.objects.create(name="Chickpeas", ingredient_group="Legumineuse")
    ing_66 = Ingredient.objects.create(
        name="Kidney beans", ingredient_group="Legumineuse"
    )
    ing_67 = Ingredient.objects.create(name="Lentils", ingredient_group="Legumineuse")
    ing_68 = Ingredient.objects.create(
        name="White beans", ingredient_group="Legumineuse"
    )
    ing_69 = Ingredient.objects.create(name="Beef cubes", ingredient_group="Meat")
    ing_70 = Ingredient.objects.create(name="Chicken", ingredient_group="Meat")
    ing_71 = Ingredient.objects.create(name="Chicken thighs", ingredient_group="Meat")
    ing_72 = Ingredient.objects.create(name="Ground beef", ingredient_group="Meat")
    ing_73 = Ingredient.objects.create(name="Ground porc", ingredient_group="Meat")
    ing_74 = Ingredient.objects.create(name="Ham", ingredient_group="Meat")
    ing_75 = Ingredient.objects.create(name="Lamb cubes", ingredient_group="Meat")
    ing_76 = Ingredient.objects.create(name="Pork sausage", ingredient_group="Meat")
    ing_77 = Ingredient.objects.create(
        name="Pork toulouse sausage", ingredient_group="Meat"
    )
    ing_78 = Ingredient.objects.create(name="Roast Beef ", ingredient_group="Meat")
    ing_79 = Ingredient.objects.create(name="stock", ingredient_group="Meat")
    ing_80 = Ingredient.objects.create(name="Turkey", ingredient_group="Meat")
    ing_81 = Ingredient.objects.create(name="Turkey thighs", ingredient_group="Meat")
    ing_82 = Ingredient.objects.create(
        name="Apple cider vinegar", ingredient_group="Oils and Sauces"
    )
    ing_83 = Ingredient.objects.create(
        name="Balsamic vinegar", ingredient_group="Oils and Sauces"
    )
    ing_84 = Ingredient.objects.create(
        name="cooking wine", ingredient_group="Oils and Sauces"
    )
    ing_85 = Ingredient.objects.create(
        name="Dijon mustard", ingredient_group="Oils and Sauces"
    )
    ing_86 = Ingredient.objects.create(name="honey", ingredient_group="Oils and Sauces")
    ing_87 = Ingredient.objects.create(
        name="maple sirop", ingredient_group="Oils and Sauces"
    )
    ing_88 = Ingredient.objects.create(
        name="Olive oil", ingredient_group="Oils and Sauces"
    )
    ing_89 = Ingredient.objects.create(
        name="Prepared mustard", ingredient_group="Oils and Sauces"
    )
    ing_90 = Ingredient.objects.create(
        name="Red wine (salt)", ingredient_group="Oils and Sauces"
    )
    ing_91 = Ingredient.objects.create(
        name="Red wine vinegar", ingredient_group="Oils and Sauces"
    )
    ing_92 = Ingredient.objects.create(
        name="Rice vinegar ", ingredient_group="Oils and Sauces"
    )
    ing_93 = Ingredient.objects.create(
        name="rice wine vinegar", ingredient_group="Oils and Sauces"
    )
    ing_94 = Ingredient.objects.create(
        name="Sesame oil", ingredient_group="Oils and Sauces"
    )
    ing_95 = Ingredient.objects.create(
        name="Soy sauce (gluten, salt)", ingredient_group="Oils and Sauces"
    )
    ing_96 = Ingredient.objects.create(
        name="Vinegar", ingredient_group="Oils and Sauces"
    )
    ing_97 = Ingredient.objects.create(
        name="White wine", ingredient_group="Oils and Sauces"
    )
    ing_98 = Ingredient.objects.create(
        name="Seafood medley (shrimp, mussels, calamari)", ingredient_group="Seafood"
    )
    ing_99 = Ingredient.objects.create(name="Shrimp", ingredient_group="Seafood")
    ing_100 = Ingredient.objects.create(name="Allspice", ingredient_group="Spices")
    ing_101 = Ingredient.objects.create(name="Bay leaves", ingredient_group="Spices")
    ing_102 = Ingredient.objects.create(name="Black pepper", ingredient_group="Spices")
    ing_103 = Ingredient.objects.create(name="Cardamom", ingredient_group="Spices")
    ing_104 = Ingredient.objects.create(name="Chili", ingredient_group="Spices")
    ing_105 = Ingredient.objects.create(name="Cinnamon", ingredient_group="Spices")
    ing_106 = Ingredient.objects.create(name="Cloves", ingredient_group="Spices")
    ing_107 = Ingredient.objects.create(name="Coriander", ingredient_group="Spices")
    ing_108 = Ingredient.objects.create(name="Cumin", ingredient_group="Spices")
    ing_109 = Ingredient.objects.create(name="Curry", ingredient_group="Spices")
    ing_110 = Ingredient.objects.create(name="Ginger powder", ingredient_group="Spices")
    ing_111 = Ingredient.objects.create(
        name="Herbes de provence", ingredient_group="Spices"
    )
    ing_112 = Ingredient.objects.create(
        name="Mustard powder", ingredient_group="Spices"
    )
    ing_113 = Ingredient.objects.create(name="Nutmeg", ingredient_group="Spices")
    ing_114 = Ingredient.objects.create(name="Paprika", ingredient_group="Spices")
    ing_115 = Ingredient.objects.create(name="Sage", ingredient_group="Spices")
    ing_116 = Ingredient.objects.create(name="Terragon", ingredient_group="Spices")
    ing_118 = Ingredient.objects.create(name="Turmeric", ingredient_group="Spices")
    ing_119 = Ingredient.objects.create(
        name="Apples", ingredient_group="Veggies & Fruits"
    )
    ing_120 = Ingredient.objects.create(
        name="Apricots", ingredient_group="Veggies & Fruits"
    )
    ing_121 = Ingredient.objects.create(
        name="Arugula", ingredient_group="Veggies & Fruits"
    )
    ing_122 = Ingredient.objects.create(
        name="Artichokes", ingredient_group="Veggies & Fruits"
    )
    ing_123 = Ingredient.objects.create(
        name="Bean sprouts", ingredient_group="Veggies & Fruits"
    )
    ing_124 = Ingredient.objects.create(
        name="Beets", ingredient_group="Veggies & Fruits"
    )
    ing_125 = Ingredient.objects.create(
        name="Bok choy", ingredient_group="Veggies & Fruits"
    )
    ing_126 = Ingredient.objects.create(
        name="Broccoli", ingredient_group="Veggies & Fruits"
    )
    ing_127 = Ingredient.objects.create(
        name="Brussel sprouts", ingredient_group="Veggies & Fruits"
    )
    ing_128 = Ingredient.objects.create(
        name="Cabbage", ingredient_group="Veggies & Fruits"
    )
    ing_129 = Ingredient.objects.create(
        name="Carrots", ingredient_group="Veggies & Fruits"
    )
    ing_130 = Ingredient.objects.create(
        name="Cauliflower", ingredient_group="Veggies & Fruits"
    )
    ing_131 = Ingredient.objects.create(
        name="celery", ingredient_group="Veggies & Fruits"
    )
    ing_132 = Ingredient.objects.create(
        name="Celery root", ingredient_group="Veggies & Fruits"
    )
    ing_133 = Ingredient.objects.create(
        name="Chicory", ingredient_group="Veggies & Fruits"
    )
    ing_134 = Ingredient.objects.create(
        name="Corn", ingredient_group="Veggies & Fruits"
    )
    ing_135 = Ingredient.objects.create(
        name="Cranberries", ingredient_group="Veggies & Fruits"
    )
    ing_136 = Ingredient.objects.create(
        name="Cucumber", ingredient_group="Veggies & Fruits"
    )
    ing_137 = Ingredient.objects.create(
        name="Eggplant", ingredient_group="Veggies & Fruits"
    )
    ing_138 = Ingredient.objects.create(
        name="Garlic", ingredient_group="Veggies & Fruits"
    )
    ing_139 = Ingredient.objects.create(
        name="Ginger", ingredient_group="Veggies & Fruits"
    )
    ing_140 = Ingredient.objects.create(
        name="Green beans", ingredient_group="Veggies & Fruits"
    )
    ing_141 = Ingredient.objects.create(
        name="Green peas", ingredient_group="Veggies & Fruits"
    )
    ing_142 = Ingredient.objects.create(
        name="Green peppers", ingredient_group="Veggies & Fruits"
    )
    ing_143 = Ingredient.objects.create(
        name="Kale", ingredient_group="Veggies & Fruits"
    )
    ing_144 = Ingredient.objects.create(
        name="Kolhrabi", ingredient_group="Veggies & Fruits"
    )
    ing_145 = Ingredient.objects.create(
        name="Leeks", ingredient_group="Veggies & Fruits"
    )
    ing_146 = Ingredient.objects.create(
        name="Lemons", ingredient_group="Veggies & Fruits"
    )
    ing_147 = Ingredient.objects.create(
        name="Lettuce", ingredient_group="Veggies & Fruits"
    )
    ing_148 = Ingredient.objects.create(
        name="lime", ingredient_group="Veggies & Fruits"
    )
    ing_149 = Ingredient.objects.create(
        name="mung bean sprouts", ingredient_group="Veggies & Fruits"
    )
    ing_150 = Ingredient.objects.create(
        name="Mushrooms", ingredient_group="Veggies & Fruits"
    )
    ing_151 = Ingredient.objects.create(
        name="Olives", ingredient_group="Veggies & Fruits"
    )
    ing_152 = Ingredient.objects.create(
        name="Onions", ingredient_group="Veggies & Fruits"
    )
    ing_153 = Ingredient.objects.create(
        name="Orange peppers", ingredient_group="Veggies & Fruits"
    )
    ing_154 = Ingredient.objects.create(
        name="parsnip", ingredient_group="Veggies & Fruits"
    )
    ing_155 = Ingredient.objects.create(
        name="Potatoes", ingredient_group="Veggies & Fruits"
    )
    ing_156 = Ingredient.objects.create(
        name="Pumpkin", ingredient_group="Veggies & Fruits"
    )
    ing_157 = Ingredient.objects.create(
        name="Radish", ingredient_group="Veggies & Fruits"
    )
    ing_158 = Ingredient.objects.create(
        name="Raisins", ingredient_group="Veggies & Fruits"
    )
    ing_159 = Ingredient.objects.create(
        name="Red peppers  ", ingredient_group="Veggies & Fruits"
    )
    ing_160 = Ingredient.objects.create(
        name="Snap peas", ingredient_group="Veggies & Fruits"
    )
    ing_161 = Ingredient.objects.create(
        name="Snow peas", ingredient_group="Veggies & Fruits"
    )
    ing_162 = Ingredient.objects.create(
        name="Spinach", ingredient_group="Veggies & Fruits"
    )
    ing_163 = Ingredient.objects.create(
        name="squash", ingredient_group="Veggies & Fruits"
    )
    ing_164 = Ingredient.objects.create(
        name="Sweet potatoes", ingredient_group="Veggies & Fruits"
    )
    ing_165 = Ingredient.objects.create(
        name="Swiss chard", ingredient_group="Veggies & Fruits"
    )
    ing_166 = Ingredient.objects.create(
        name="Tomatoes", ingredient_group="Veggies & Fruits"
    )
    ing_167 = Ingredient.objects.create(
        name="Turnips", ingredient_group="Veggies & Fruits"
    )
    ing_168 = Ingredient.objects.create(
        name="Yellow beans", ingredient_group="Veggies & Fruits"
    )
    ing_169 = Ingredient.objects.create(
        name="zucchini", ingredient_group="Veggies & Fruits"
    )
    #
    print_rows(Ingredient.objects)

    # -----------------------------------------------------------------------------
    com_1 = Component.objects.create(
        name="Beef bourguignon", component_group="main_dish"
    )
    com_2 = Component.objects.create(name="Beef meatballs", component_group="main_dish")
    com_3 = Component.objects.create(name="Beef meatloaf", component_group="main_dish")
    com_4 = Component.objects.create(
        name="Beet and apple salad", component_group="main_dish"
    )
    com_5 = Component.objects.create(
        name="Chicken and squash curry", component_group="main_dish"
    )
    com_6 = Component.objects.create(
        name="Chicken cacciatore", component_group="main_dish"
    )
    com_7 = Component.objects.create(
        name="Chicken pot pie", component_group="main_dish"
    )
    com_8 = Component.objects.create(
        name="Chicken, vegetable and quinoa couscous", component_group="main_dish"
    )
    com_9 = Component.objects.create(name="Cod cakes", component_group="main_dish")
    com_10 = Component.objects.create(name="Coq au vin", component_group="main_dish")
    com_11 = Component.objects.create(
        name="Creamy salmon pasta", component_group="main_dish"
    )
    com_12 = Component.objects.create(
        name="Egg and salmon salad", component_group="main_dish"
    )
    com_13 = Component.objects.create(
        name="Eggplant parmesan", component_group="main_dish"
    )
    com_14 = Component.objects.create(name="Fish chowder", component_group="main_dish")
    com_15 = Component.objects.create(name="Ginger pork", component_group="main_dish")
    com_16 = Component.objects.create(
        name="Lemon baked haddock", component_group="main_dish"
    )
    com_17 = Component.objects.create(
        name="Macaroni and cheese", component_group="main_dish"
    )
    com_18 = Component.objects.create(name="Meat pie", component_group="main_dish")
    com_19 = Component.objects.create(name="Moussaka", component_group="main_dish")
    com_20 = Component.objects.create(
        name="Mushroom rice casserole", component_group="main_dish"
    )
    com_21 = Component.objects.create(name="Niçoise salad", component_group="main_dish")
    com_22 = Component.objects.create(
        name="Peanut chicken with rice noodles", component_group="main_dish"
    )
    com_23 = Component.objects.create(
        name="Penne with pesto and feta", component_group="main_dish"
    )
    com_24 = Component.objects.create(name="Roast beef", component_group="main_dish")
    com_25 = Component.objects.create(
        name="Salmon and egg salad", component_group="main_dish"
    )
    com_26 = Component.objects.create(
        name="Sausage and cabbage", component_group="main_dish"
    )
    com_27 = Component.objects.create(
        name="Sausage cassoulet", component_group="main_dish"
    )
    com_28 = Component.objects.create(name="Seafood pilaf", component_group="main_dish")
    com_29 = Component.objects.create(
        name="Shepherd's pie", component_group="main_dish"
    )
    com_30 = Component.objects.create(
        name="Shrimp and mushroom linguini", component_group="main_dish"
    )
    com_31 = Component.objects.create(
        name="Shrimp stir-fry", component_group="main_dish"
    )
    com_32 = Component.objects.create(
        name="Spaghetti with meat sauce", component_group="main_dish"
    )
    com_33 = Component.objects.create(
        name="Turkey with mushroom sauce", component_group="main_dish"
    )
    com_34 = Component.objects.create(
        name="Vegatable and chickpea curry", component_group="main_dish"
    )
    com_35 = Component.objects.create(
        name="Vegetable fried rice", component_group="main_dish"
    )
    com_36 = Component.objects.create(
        name="Vegetable lasagna", component_group="main_dish"
    )
    com_37 = Component.objects.create(
        name="Vegetarian chili", component_group="main_dish"
    )
    com_38 = Component.objects.create(
        name="Vegetarian quiche", component_group="main_dish"
    )
    com_39 = Component.objects.create(
        name="Walnut and cranberry rice salad", component_group="main_dish"
    )
    #
    print_rows(Component.objects)

    # -----------------------------------------------------------------------------
    com_ing_1 = Component_ingredient.objects.create(component=com_1, ingredient=ing_101)
    com_ing_2 = Component_ingredient.objects.create(component=com_1, ingredient=ing_69)
    com_ing_3 = Component_ingredient.objects.create(component=com_1, ingredient=ing_129)
    com_ing_4 = Component_ingredient.objects.create(component=com_1, ingredient=ing_138)
    com_ing_5 = Component_ingredient.objects.create(component=com_1, ingredient=ing_150)
    com_ing_6 = Component_ingredient.objects.create(component=com_1, ingredient=ing_152)
    com_ing_7 = Component_ingredient.objects.create(component=com_1, ingredient=ing_49)
    com_ing_8 = Component_ingredient.objects.create(component=com_1, ingredient=ing_90)
    com_ing_9 = Component_ingredient.objects.create(component=com_1, ingredient=ing_51)
    com_ing_10 = Component_ingredient.objects.create(
        component=com_1, ingredient=ing_166
    )
    com_ing_11 = Component_ingredient.objects.create(component=com_1, ingredient=ing_61)
    com_ing_12 = Component_ingredient.objects.create(component=com_2, ingredient=ing_54)
    com_ing_13 = Component_ingredient.objects.create(component=com_2, ingredient=ing_9)
    com_ing_14 = Component_ingredient.objects.create(component=com_2, ingredient=ing_11)
    com_ing_15 = Component_ingredient.objects.create(
        component=com_2, ingredient=ing_138
    )
    com_ing_16 = Component_ingredient.objects.create(component=com_2, ingredient=ing_72)
    com_ing_17 = Component_ingredient.objects.create(component=com_2, ingredient=ing_13)
    com_ing_18 = Component_ingredient.objects.create(
        component=com_2, ingredient=ing_152
    )
    com_ing_19 = Component_ingredient.objects.create(component=com_2, ingredient=ing_48)
    com_ing_20 = Component_ingredient.objects.create(
        component=com_2, ingredient=ing_166
    )
    com_ing_21 = Component_ingredient.objects.create(
        component=com_2, ingredient=ing_169
    )
    com_ing_22 = Component_ingredient.objects.create(component=com_3, ingredient=ing_54)
    com_ing_23 = Component_ingredient.objects.create(component=com_3, ingredient=ing_55)
    com_ing_24 = Component_ingredient.objects.create(component=com_3, ingredient=ing_11)
    com_ing_25 = Component_ingredient.objects.create(
        component=com_3, ingredient=ing_138
    )
    com_ing_26 = Component_ingredient.objects.create(component=com_3, ingredient=ing_72)
    com_ing_27 = Component_ingredient.objects.create(component=com_3, ingredient=ing_13)
    com_ing_28 = Component_ingredient.objects.create(
        component=com_3, ingredient=ing_152
    )
    com_ing_29 = Component_ingredient.objects.create(
        component=com_3, ingredient=ing_166
    )
    com_ing_30 = Component_ingredient.objects.create(
        component=com_4, ingredient=ing_119
    )
    com_ing_31 = Component_ingredient.objects.create(
        component=com_4, ingredient=ing_124
    )
    com_ing_32 = Component_ingredient.objects.create(component=com_4, ingredient=ing_2)
    com_ing_33 = Component_ingredient.objects.create(component=com_4, ingredient=ing_85)
    com_ing_34 = Component_ingredient.objects.create(
        component=com_4, ingredient=ing_138
    )
    com_ing_35 = Component_ingredient.objects.create(
        component=com_4, ingredient=ing_147
    )
    com_ing_36 = Component_ingredient.objects.create(component=com_4, ingredient=ing_87)
    com_ing_37 = Component_ingredient.objects.create(component=com_4, ingredient=ing_88)
    com_ing_38 = Component_ingredient.objects.create(
        component=com_4, ingredient=ing_152
    )
    com_ing_39 = Component_ingredient.objects.create(component=com_5, ingredient=ing_70)
    com_ing_40 = Component_ingredient.objects.create(
        component=com_5, ingredient=ing_129
    )
    com_ing_41 = Component_ingredient.objects.create(
        component=com_5, ingredient=ing_138
    )
    com_ing_42 = Component_ingredient.objects.create(
        component=com_5, ingredient=ing_139
    )
    com_ing_43 = Component_ingredient.objects.create(
        component=com_5, ingredient=ing_152
    )
    com_ing_41 = Component_ingredient.objects.create(
        component=com_5, ingredient=ing_107
    )
    com_ing_42 = Component_ingredient.objects.create(
        component=com_5, ingredient=ing_108
    )
    com_ing_43 = Component_ingredient.objects.create(
        component=com_5, ingredient=ing_118
    )
    com_ing_43 = Component_ingredient.objects.create(
        component=com_5, ingredient=ing_112
    )
    com_ing_44 = Component_ingredient.objects.create(
        component=com_5, ingredient=ing_110
    )
    com_ing_45 = Component_ingredient.objects.create(
        component=com_5, ingredient=ing_163
    )
    com_ing_46 = Component_ingredient.objects.create(
        component=com_5, ingredient=ing_166
    )
    com_ing_47 = Component_ingredient.objects.create(component=com_5, ingredient=ing_15)
    com_ing_48 = Component_ingredient.objects.create(
        component=com_5, ingredient=ing_169
    )
    com_ing_49 = Component_ingredient.objects.create(component=com_6, ingredient=ing_55)
    com_ing_50 = Component_ingredient.objects.create(
        component=com_6, ingredient=ing_131
    )
    com_ing_51 = Component_ingredient.objects.create(component=com_6, ingredient=ing_71)
    com_ing_52 = Component_ingredient.objects.create(component=com_6, ingredient=ing_84)
    com_ing_53 = Component_ingredient.objects.create(
        component=com_6, ingredient=ing_138
    )
    com_ing_54 = Component_ingredient.objects.create(
        component=com_6, ingredient=ing_152
    )
    com_ing_55 = Component_ingredient.objects.create(component=com_6, ingredient=ing_50)
    com_ing_56 = Component_ingredient.objects.create(component=com_6, ingredient=ing_79)
    com_ing_57 = Component_ingredient.objects.create(component=com_6, ingredient=ing_31)
    com_ing_58 = Component_ingredient.objects.create(
        component=com_6, ingredient=ing_166
    )
    com_ing_59 = Component_ingredient.objects.create(
        component=com_7, ingredient=ing_129
    )
    com_ing_60 = Component_ingredient.objects.create(
        component=com_7, ingredient=ing_131
    )
    com_ing_61 = Component_ingredient.objects.create(component=com_7, ingredient=ing_71)
    com_ing_62 = Component_ingredient.objects.create(component=com_7, ingredient=ing_11)
    com_ing_63 = Component_ingredient.objects.create(
        component=com_7, ingredient=ing_138
    )
    com_ing_64 = Component_ingredient.objects.create(component=com_7, ingredient=ing_13)
    com_ing_65 = Component_ingredient.objects.create(
        component=com_7, ingredient=ing_152
    )
    com_ing_66 = Component_ingredient.objects.create(component=com_7, ingredient=ing_49)
    com_ing_67 = Component_ingredient.objects.create(component=com_7, ingredient=ing_61)
    com_ing_68 = Component_ingredient.objects.create(
        component=com_8, ingredient=ing_100
    )
    com_ing_69 = Component_ingredient.objects.create(
        component=com_8, ingredient=ing_103
    )
    com_ing_70 = Component_ingredient.objects.create(component=com_8, ingredient=ing_71)
    com_ing_71 = Component_ingredient.objects.create(
        component=com_8, ingredient=ing_105
    )
    com_ing_72 = Component_ingredient.objects.create(
        component=com_8, ingredient=ing_106
    )
    com_ing_73 = Component_ingredient.objects.create(
        component=com_8, ingredient=ing_107
    )
    com_ing_74 = Component_ingredient.objects.create(component=com_8, ingredient=ing_57)
    com_ing_75 = Component_ingredient.objects.create(
        component=com_8, ingredient=ing_137
    )
    com_ing_76 = Component_ingredient.objects.create(
        component=com_8, ingredient=ing_138
    )
    com_ing_77 = Component_ingredient.objects.create(
        component=com_8, ingredient=ing_139
    )
    com_ing_78 = Component_ingredient.objects.create(
        component=com_8, ingredient=ing_143
    )
    com_ing_79 = Component_ingredient.objects.create(
        component=com_8, ingredient=ing_146
    )
    com_ing_80 = Component_ingredient.objects.create(
        component=com_8, ingredient=ing_152
    )
    com_ing_81 = Component_ingredient.objects.create(
        component=com_8, ingredient=ing_114
    )
    com_ing_82 = Component_ingredient.objects.create(component=com_8, ingredient=ing_60)
    com_ing_83 = Component_ingredient.objects.create(
        component=com_8, ingredient=ing_163
    )
    com_ing_84 = Component_ingredient.objects.create(
        component=com_8, ingredient=ing_166
    )
    com_ing_85 = Component_ingredient.objects.create(component=com_9, ingredient=ing_54)
    com_ing_86 = Component_ingredient.objects.create(component=com_9, ingredient=ing_40)
    com_ing_87 = Component_ingredient.objects.create(component=com_9, ingredient=ing_46)
    com_ing_88 = Component_ingredient.objects.create(component=com_9, ingredient=ing_11)
    com_ing_89 = Component_ingredient.objects.create(
        component=com_9, ingredient=ing_138
    )
    com_ing_90 = Component_ingredient.objects.create(
        component=com_9, ingredient=ing_146
    )
    com_ing_91 = Component_ingredient.objects.create(component=com_9, ingredient=ing_49)
    com_ing_92 = Component_ingredient.objects.create(
        component=com_9, ingredient=ing_155
    )
    com_ing_93 = Component_ingredient.objects.create(component=com_9, ingredient=ing_15)
    com_ing_94 = Component_ingredient.objects.create(
        component=com_10, ingredient=ing_129
    )
    com_ing_95 = Component_ingredient.objects.create(
        component=com_10, ingredient=ing_71
    )
    com_ing_96 = Component_ingredient.objects.create(
        component=com_10, ingredient=ing_84
    )
    com_ing_97 = Component_ingredient.objects.create(
        component=com_10, ingredient=ing_138
    )
    com_ing_98 = Component_ingredient.objects.create(
        component=com_10, ingredient=ing_150
    )
    com_ing_99 = Component_ingredient.objects.create(
        component=com_10, ingredient=ing_152
    )
    com_ing_100 = Component_ingredient.objects.create(
        component=com_10, ingredient=ing_49
    )
    com_ing_101 = Component_ingredient.objects.create(
        component=com_10, ingredient=ing_51
    )
    com_ing_102 = Component_ingredient.objects.create(
        component=com_10, ingredient=ing_61
    )
    com_ing_103 = Component_ingredient.objects.create(
        component=com_11, ingredient=ing_1
    )
    com_ing_104 = Component_ingredient.objects.create(
        component=com_11, ingredient=ing_38
    )
    com_ing_105 = Component_ingredient.objects.create(
        component=com_11, ingredient=ing_129
    )
    com_ing_106 = Component_ingredient.objects.create(
        component=com_11, ingredient=ing_46
    )
    com_ing_107 = Component_ingredient.objects.create(
        component=com_11, ingredient=ing_138
    )
    com_ing_108 = Component_ingredient.objects.create(
        component=com_11, ingredient=ing_146
    )
    com_ing_109 = Component_ingredient.objects.create(
        component=com_11, ingredient=ing_13
    )
    com_ing_110 = Component_ingredient.objects.create(
        component=com_11, ingredient=ing_152
    )
    com_ing_111 = Component_ingredient.objects.create(
        component=com_11, ingredient=ing_166
    )
    com_ing_112 = Component_ingredient.objects.create(
        component=com_11, ingredient=ing_61
    )
    com_ing_113 = Component_ingredient.objects.create(
        component=com_11, ingredient=ing_37
    )
    com_ing_114 = Component_ingredient.objects.create(
        component=com_11, ingredient=ing_169
    )
    com_ing_115 = Component_ingredient.objects.create(
        component=com_12, ingredient=ing_38
    )
    com_ing_116 = Component_ingredient.objects.create(
        component=com_12, ingredient=ing_129
    )
    com_ing_117 = Component_ingredient.objects.create(
        component=com_12, ingredient=ing_46
    )
    com_ing_118 = Component_ingredient.objects.create(
        component=com_12, ingredient=ing_11
    )
    com_ing_119 = Component_ingredient.objects.create(
        component=com_12, ingredient=ing_146
    )
    com_ing_120 = Component_ingredient.objects.create(
        component=com_12, ingredient=ing_147
    )
    com_ing_121 = Component_ingredient.objects.create(
        component=com_12, ingredient=ing_116
    )
    com_ing_122 = Component_ingredient.objects.create(
        component=com_13, ingredient=ing_129
    )
    com_ing_123 = Component_ingredient.objects.create(
        component=com_13, ingredient=ing_131
    )
    com_ing_124 = Component_ingredient.objects.create(
        component=com_13, ingredient=ing_4
    )
    com_ing_125 = Component_ingredient.objects.create(
        component=com_13, ingredient=ing_9
    )
    com_ing_126 = Component_ingredient.objects.create(
        component=com_13, ingredient=ing_137
    )
    com_ing_127 = Component_ingredient.objects.create(
        component=com_13, ingredient=ing_138
    )
    com_ing_128 = Component_ingredient.objects.create(
        component=com_13, ingredient=ing_152
    )
    com_ing_129 = Component_ingredient.objects.create(
        component=com_13, ingredient=ing_90
    )
    com_ing_130 = Component_ingredient.objects.create(
        component=com_13, ingredient=ing_166
    )
    com_ing_131 = Component_ingredient.objects.create(
        component=com_14, ingredient=ing_102
    )
    com_ing_132 = Component_ingredient.objects.create(
        component=com_14, ingredient=ing_1
    )
    com_ing_133 = Component_ingredient.objects.create(
        component=com_14, ingredient=ing_134
    )
    com_ing_134 = Component_ingredient.objects.create(
        component=com_14, ingredient=ing_138
    )
    com_ing_135 = Component_ingredient.objects.create(
        component=com_14, ingredient=ing_41
    )
    com_ing_136 = Component_ingredient.objects.create(
        component=com_14, ingredient=ing_13
    )
    com_ing_137 = Component_ingredient.objects.create(
        component=com_14, ingredient=ing_152
    )
    com_ing_138 = Component_ingredient.objects.create(
        component=com_14, ingredient=ing_155
    )
    com_ing_139 = Component_ingredient.objects.create(
        component=com_14, ingredient=ing_51
    )
    com_ing_140 = Component_ingredient.objects.create(
        component=com_14, ingredient=ing_61
    )
    com_ing_141 = Component_ingredient.objects.create(
        component=com_15, ingredient=ing_129
    )
    com_ing_142 = Component_ingredient.objects.create(
        component=com_15, ingredient=ing_20
    )
    com_ing_143 = Component_ingredient.objects.create(
        component=com_15, ingredient=ing_138
    )
    com_ing_144 = Component_ingredient.objects.create(
        component=com_15, ingredient=ing_139
    )
    com_ing_145 = Component_ingredient.objects.create(
        component=com_15, ingredient=ing_73
    )
    com_ing_146 = Component_ingredient.objects.create(
        component=com_15, ingredient=ing_152
    )
    com_ing_147 = Component_ingredient.objects.create(
        component=com_15, ingredient=ing_28
    )
    com_ing_148 = Component_ingredient.objects.create(
        component=com_15, ingredient=ing_93
    )
    com_ing_149 = Component_ingredient.objects.create(
        component=com_15, ingredient=ing_94
    )
    com_ing_150 = Component_ingredient.objects.create(
        component=com_15, ingredient=ing_29
    )
    com_ing_151 = Component_ingredient.objects.create(
        component=com_15, ingredient=ing_95
    )
    com_ing_152 = Component_ingredient.objects.create(
        component=com_15, ingredient=ing_169
    )
    com_ing_153 = Component_ingredient.objects.create(
        component=com_16, ingredient=ing_102
    )
    com_ing_154 = Component_ingredient.objects.create(
        component=com_16, ingredient=ing_54
    )
    com_ing_155 = Component_ingredient.objects.create(
        component=com_16, ingredient=ing_4
    )
    com_ing_156 = Component_ingredient.objects.create(
        component=com_16, ingredient=ing_11
    )
    com_ing_157 = Component_ingredient.objects.create(
        component=com_16, ingredient=ing_138
    )
    com_ing_158 = Component_ingredient.objects.create(
        component=com_16, ingredient=ing_41
    )
    com_ing_159 = Component_ingredient.objects.create(
        component=com_16, ingredient=ing_146
    )
    com_ing_160 = Component_ingredient.objects.create(
        component=com_16, ingredient=ing_13
    )
    com_ing_161 = Component_ingredient.objects.create(
        component=com_16, ingredient=ing_152
    )
    com_ing_162 = Component_ingredient.objects.create(
        component=com_16, ingredient=ing_155
    )
    com_ing_163 = Component_ingredient.objects.create(
        component=com_16, ingredient=ing_116
    )
    com_ing_164 = Component_ingredient.objects.create(
        component=com_17, ingredient=ing_54
    )
    com_ing_165 = Component_ingredient.objects.create(
        component=com_17, ingredient=ing_1
    )
    com_ing_166 = Component_ingredient.objects.create(
        component=com_17, ingredient=ing_130
    )
    com_ing_167 = Component_ingredient.objects.create(
        component=com_17, ingredient=ing_4
    )
    com_ing_168 = Component_ingredient.objects.create(
        component=com_17, ingredient=ing_9
    )
    com_ing_169 = Component_ingredient.objects.create(
        component=com_17, ingredient=ing_44
    )
    com_ing_170 = Component_ingredient.objects.create(
        component=com_17, ingredient=ing_138
    )
    com_ing_171 = Component_ingredient.objects.create(
        component=com_17, ingredient=ing_141
    )
    com_ing_172 = Component_ingredient.objects.create(
        component=com_17, ingredient=ing_13
    )
    com_ing_173 = Component_ingredient.objects.create(
        component=com_17, ingredient=ing_150
    )
    com_ing_174 = Component_ingredient.objects.create(
        component=com_17, ingredient=ing_112
    )
    com_ing_175 = Component_ingredient.objects.create(
        component=com_17, ingredient=ing_113
    )
    com_ing_176 = Component_ingredient.objects.create(
        component=com_17, ingredient=ing_152
    )
    com_ing_177 = Component_ingredient.objects.create(
        component=com_17, ingredient=ing_49
    )
    com_ing_178 = Component_ingredient.objects.create(
        component=com_17, ingredient=ing_32
    )
    com_ing_179 = Component_ingredient.objects.create(
        component=com_17, ingredient=ing_61
    )
    com_ing_180 = Component_ingredient.objects.create(
        component=com_17, ingredient=ing_35
    )
    com_ing_181 = Component_ingredient.objects.create(
        component=com_18, ingredient=ing_102
    )
    com_ing_182 = Component_ingredient.objects.create(
        component=com_18, ingredient=ing_54
    )
    com_ing_183 = Component_ingredient.objects.create(
        component=com_18, ingredient=ing_105
    )
    com_ing_184 = Component_ingredient.objects.create(
        component=com_18, ingredient=ing_138
    )
    com_ing_185 = Component_ingredient.objects.create(
        component=com_18, ingredient=ing_72
    )
    com_ing_186 = Component_ingredient.objects.create(
        component=com_18, ingredient=ing_73
    )
    com_ing_187 = Component_ingredient.objects.create(
        component=com_18, ingredient=ing_113
    )
    com_ing_188 = Component_ingredient.objects.create(
        component=com_18, ingredient=ing_152
    )
    com_ing_189 = Component_ingredient.objects.create(
        component=com_18, ingredient=ing_155
    )
    com_ing_190 = Component_ingredient.objects.create(
        component=com_18, ingredient=ing_61
    )
    com_ing_191 = Component_ingredient.objects.create(
        component=com_19, ingredient=ing_100
    )
    com_ing_192 = Component_ingredient.objects.create(
        component=com_19, ingredient=ing_1
    )
    com_ing_193 = Component_ingredient.objects.create(
        component=com_19, ingredient=ing_9
    )
    com_ing_194 = Component_ingredient.objects.create(
        component=com_19, ingredient=ing_105
    )
    com_ing_195 = Component_ingredient.objects.create(
        component=com_19, ingredient=ing_137
    )
    com_ing_196 = Component_ingredient.objects.create(
        component=com_19, ingredient=ing_11
    )
    com_ing_197 = Component_ingredient.objects.create(
        component=com_19, ingredient=ing_138
    )
    com_ing_198 = Component_ingredient.objects.create(
        component=com_19, ingredient=ing_72
    )
    com_ing_199 = Component_ingredient.objects.create(
        component=com_19, ingredient=ing_13
    )
    com_ing_200 = Component_ingredient.objects.create(
        component=com_19, ingredient=ing_152
    )
    com_ing_201 = Component_ingredient.objects.create(
        component=com_19, ingredient=ing_48
    )
    com_ing_202 = Component_ingredient.objects.create(
        component=com_19, ingredient=ing_166
    )
    com_ing_203 = Component_ingredient.objects.create(
        component=com_19, ingredient=ing_61
    )
    com_ing_204 = Component_ingredient.objects.create(
        component=com_20, ingredient=ing_102
    )
    com_ing_205 = Component_ingredient.objects.create(
        component=com_20, ingredient=ing_55
    )
    com_ing_206 = Component_ingredient.objects.create(
        component=com_20, ingredient=ing_129
    )
    com_ing_207 = Component_ingredient.objects.create(
        component=com_20, ingredient=ing_9
    )
    com_ing_208 = Component_ingredient.objects.create(
        component=com_20, ingredient=ing_138
    )
    com_ing_209 = Component_ingredient.objects.create(
        component=com_20, ingredient=ing_145
    )
    com_ing_210 = Component_ingredient.objects.create(
        component=com_20, ingredient=ing_47
    )
    com_ing_211 = Component_ingredient.objects.create(
        component=com_20, ingredient=ing_150
    )
    com_ing_212 = Component_ingredient.objects.create(
        component=com_20, ingredient=ing_49
    )
    com_ing_213 = Component_ingredient.objects.create(
        component=com_20, ingredient=ing_116
    )
    com_ing_214 = Component_ingredient.objects.create(
        component=com_20, ingredient=ing_32
    )
    com_ing_215 = Component_ingredient.objects.create(
        component=com_20, ingredient=ing_97
    )
    com_ing_216 = Component_ingredient.objects.create(
        component=com_20, ingredient=ing_63
    )
    com_ing_217 = Component_ingredient.objects.create(
        component=com_20, ingredient=ing_169
    )
    com_ing_218 = Component_ingredient.objects.create(
        component=com_21, ingredient=ing_39
    )
    com_ing_219 = Component_ingredient.objects.create(
        component=com_21, ingredient=ing_129
    )
    com_ing_220 = Component_ingredient.objects.create(
        component=com_21, ingredient=ing_85
    )
    com_ing_221 = Component_ingredient.objects.create(
        component=com_21, ingredient=ing_138
    )
    com_ing_222 = Component_ingredient.objects.create(
        component=com_21, ingredient=ing_140
    )
    com_ing_223 = Component_ingredient.objects.create(
        component=com_21, ingredient=ing_146
    )
    com_ing_224 = Component_ingredient.objects.create(
        component=com_21, ingredient=ing_151
    )
    com_ing_225 = Component_ingredient.objects.create(
        component=com_21, ingredient=ing_152
    )
    com_ing_226 = Component_ingredient.objects.create(
        component=com_21, ingredient=ing_116
    )
    com_ing_227 = Component_ingredient.objects.create(
        component=com_22, ingredient=ing_129
    )
    com_ing_228 = Component_ingredient.objects.create(
        component=com_22, ingredient=ing_71
    )
    com_ing_229 = Component_ingredient.objects.create(
        component=com_22, ingredient=ing_45
    )
    com_ing_230 = Component_ingredient.objects.create(
        component=com_22, ingredient=ing_107
    )
    com_ing_231 = Component_ingredient.objects.create(
        component=com_22, ingredient=ing_138
    )
    com_ing_232 = Component_ingredient.objects.create(
        component=com_22, ingredient=ing_139
    )
    com_ing_233 = Component_ingredient.objects.create(
        component=com_22, ingredient=ing_86
    )
    com_ing_234 = Component_ingredient.objects.create(
        component=com_22, ingredient=ing_148
    )
    com_ing_235 = Component_ingredient.objects.create(
        component=com_22, ingredient=ing_152
    )
    com_ing_236 = Component_ingredient.objects.create(
        component=com_22, ingredient=ing_24
    )
    com_ing_237 = Component_ingredient.objects.create(
        component=com_22, ingredient=ing_25
    )
    com_ing_238 = Component_ingredient.objects.create(
        component=com_22, ingredient=ing_28
    )
    com_ing_239 = Component_ingredient.objects.create(
        component=com_22, ingredient=ing_94
    )
    com_ing_240 = Component_ingredient.objects.create(
        component=com_22, ingredient=ing_95
    )
    com_ing_241 = Component_ingredient.objects.create(
        component=com_22, ingredient=ing_118
    )
    com_ing_242 = Component_ingredient.objects.create(
        component=com_23, ingredient=ing_43
    )
    com_ing_243 = Component_ingredient.objects.create(
        component=com_23, ingredient=ing_129
    )
    com_ing_244 = Component_ingredient.objects.create(
        component=com_23, ingredient=ing_6
    )
    com_ing_245 = Component_ingredient.objects.create(
        component=com_23, ingredient=ing_9
    )
    com_ing_246 = Component_ingredient.objects.create(
        component=com_23, ingredient=ing_138
    )
    com_ing_247 = Component_ingredient.objects.create(
        component=com_23, ingredient=ing_146
    )
    com_ing_248 = Component_ingredient.objects.create(
        component=com_23, ingredient=ing_147
    )
    com_ing_249 = Component_ingredient.objects.create(
        component=com_23, ingredient=ing_152
    )
    com_ing_250 = Component_ingredient.objects.create(
        component=com_23, ingredient=ing_32
    )
    com_ing_251 = Component_ingredient.objects.create(
        component=com_23, ingredient=ing_36
    )
    com_ing_252 = Component_ingredient.objects.create(
        component=com_24, ingredient=ing_129
    )
    com_ing_253 = Component_ingredient.objects.create(
        component=com_24, ingredient=ing_131
    )
    com_ing_254 = Component_ingredient.objects.create(
        component=com_24, ingredient=ing_85
    )
    com_ing_255 = Component_ingredient.objects.create(
        component=com_24, ingredient=ing_138
    )
    com_ing_256 = Component_ingredient.objects.create(
        component=com_24, ingredient=ing_150
    )
    com_ing_257 = Component_ingredient.objects.create(
        component=com_24, ingredient=ing_152
    )
    com_ing_258 = Component_ingredient.objects.create(
        component=com_24, ingredient=ing_78
    )
    com_ing_259 = Component_ingredient.objects.create(
        component=com_25, ingredient=ing_38
    )
    com_ing_260 = Component_ingredient.objects.create(
        component=com_25, ingredient=ing_129
    )
    com_ing_261 = Component_ingredient.objects.create(
        component=com_25, ingredient=ing_46
    )
    com_ing_262 = Component_ingredient.objects.create(
        component=com_25, ingredient=ing_11
    )
    com_ing_263 = Component_ingredient.objects.create(
        component=com_25, ingredient=ing_138
    )
    com_ing_264 = Component_ingredient.objects.create(
        component=com_25, ingredient=ing_146
    )
    com_ing_265 = Component_ingredient.objects.create(
        component=com_25, ingredient=ing_147
    )
    com_ing_266 = Component_ingredient.objects.create(
        component=com_25, ingredient=ing_88
    )
    com_ing_267 = Component_ingredient.objects.create(
        component=com_25, ingredient=ing_152
    )
    com_ing_268 = Component_ingredient.objects.create(
        component=com_26, ingredient=ing_102
    )
    com_ing_269 = Component_ingredient.objects.create(
        component=com_26, ingredient=ing_128
    )
    com_ing_270 = Component_ingredient.objects.create(
        component=com_26, ingredient=ing_138
    )
    com_ing_271 = Component_ingredient.objects.create(
        component=com_26, ingredient=ing_152
    )
    com_ing_272 = Component_ingredient.objects.create(
        component=com_26, ingredient=ing_154
    )
    com_ing_273 = Component_ingredient.objects.create(
        component=com_26, ingredient=ing_77
    )
    com_ing_274 = Component_ingredient.objects.create(
        component=com_26, ingredient=ing_155
    )
    com_ing_275 = Component_ingredient.objects.create(
        component=com_26, ingredient=ing_97
    )
    com_ing_276 = Component_ingredient.objects.create(
        component=com_27, ingredient=ing_101
    )
    com_ing_277 = Component_ingredient.objects.create(
        component=com_27, ingredient=ing_129
    )
    com_ing_278 = Component_ingredient.objects.create(
        component=com_27, ingredient=ing_131
    )
    com_ing_279 = Component_ingredient.objects.create(
        component=com_27, ingredient=ing_138
    )
    com_ing_280 = Component_ingredient.objects.create(
        component=com_27, ingredient=ing_152
    )
    com_ing_281 = Component_ingredient.objects.create(
        component=com_27, ingredient=ing_49
    )
    com_ing_282 = Component_ingredient.objects.create(
        component=com_27, ingredient=ing_77
    )
    com_ing_283 = Component_ingredient.objects.create(
        component=com_27, ingredient=ing_51
    )
    com_ing_284 = Component_ingredient.objects.create(
        component=com_27, ingredient=ing_166
    )
    com_ing_285 = Component_ingredient.objects.create(
        component=com_27, ingredient=ing_68
    )
    com_ing_286 = Component_ingredient.objects.create(
        component=com_28, ingredient=ing_55
    )
    com_ing_287 = Component_ingredient.objects.create(
        component=com_28, ingredient=ing_131
    )
    com_ing_288 = Component_ingredient.objects.create(
        component=com_28, ingredient=ing_138
    )
    com_ing_289 = Component_ingredient.objects.create(
        component=com_28, ingredient=ing_152
    )
    com_ing_290 = Component_ingredient.objects.create(
        component=com_28, ingredient=ing_114
    )
    com_ing_291 = Component_ingredient.objects.create(
        component=com_28, ingredient=ing_49
    )
    com_ing_292 = Component_ingredient.objects.create(
        component=com_28, ingredient=ing_98
    )
    com_ing_293 = Component_ingredient.objects.create(
        component=com_28, ingredient=ing_166
    )
    com_ing_294 = Component_ingredient.objects.create(
        component=com_28, ingredient=ing_97
    )
    com_ing_295 = Component_ingredient.objects.create(
        component=com_29, ingredient=ing_54
    )
    com_ing_296 = Component_ingredient.objects.create(
        component=com_29, ingredient=ing_105
    )
    com_ing_297 = Component_ingredient.objects.create(
        component=com_29, ingredient=ing_106
    )
    com_ing_298 = Component_ingredient.objects.create(
        component=com_29, ingredient=ing_134
    )
    com_ing_299 = Component_ingredient.objects.create(
        component=com_29, ingredient=ing_138
    )
    com_ing_300 = Component_ingredient.objects.create(
        component=com_29, ingredient=ing_72
    )
    com_ing_301 = Component_ingredient.objects.create(
        component=com_29, ingredient=ing_13
    )
    com_ing_302 = Component_ingredient.objects.create(
        component=com_29, ingredient=ing_113
    )
    com_ing_303 = Component_ingredient.objects.create(
        component=com_29, ingredient=ing_152
    )
    com_ing_304 = Component_ingredient.objects.create(
        component=com_29, ingredient=ing_114
    )
    com_ing_305 = Component_ingredient.objects.create(
        component=com_29, ingredient=ing_155
    )
    com_ing_306 = Component_ingredient.objects.create(
        component=com_29, ingredient=ing_115
    )
    com_ing_307 = Component_ingredient.objects.create(
        component=com_29, ingredient=ing_51
    )
    com_ing_308 = Component_ingredient.objects.create(
        component=com_30, ingredient=ing_1
    )
    com_ing_309 = Component_ingredient.objects.create(
        component=com_30, ingredient=ing_138
    )
    com_ing_310 = Component_ingredient.objects.create(
        component=com_30, ingredient=ing_13
    )
    com_ing_311 = Component_ingredient.objects.create(
        component=com_30, ingredient=ing_152
    )
    com_ing_312 = Component_ingredient.objects.create(
        component=com_30, ingredient=ing_99
    )
    com_ing_313 = Component_ingredient.objects.create(
        component=com_30, ingredient=ing_116
    )
    com_ing_314 = Component_ingredient.objects.create(
        component=com_30, ingredient=ing_166
    )
    com_ing_315 = Component_ingredient.objects.create(
        component=com_30, ingredient=ing_61
    )
    com_ing_316 = Component_ingredient.objects.create(
        component=com_31, ingredient=ing_126
    )
    com_ing_317 = Component_ingredient.objects.create(
        component=com_31, ingredient=ing_129
    )
    com_ing_318 = Component_ingredient.objects.create(
        component=com_31, ingredient=ing_138
    )
    com_ing_319 = Component_ingredient.objects.create(
        component=com_31, ingredient=ing_139
    )
    com_ing_320 = Component_ingredient.objects.create(
        component=com_31, ingredient=ing_142
    )
    com_ing_321 = Component_ingredient.objects.create(
        component=com_31, ingredient=ing_149
    )
    com_ing_322 = Component_ingredient.objects.create(
        component=com_31, ingredient=ing_150
    )
    com_ing_323 = Component_ingredient.objects.create(
        component=com_31, ingredient=ing_152
    )
    com_ing_324 = Component_ingredient.objects.create(
        component=com_31, ingredient=ing_159
    )
    com_ing_325 = Component_ingredient.objects.create(
        component=com_31, ingredient=ing_28
    )
    com_ing_326 = Component_ingredient.objects.create(
        component=com_31, ingredient=ing_94
    )
    com_ing_327 = Component_ingredient.objects.create(
        component=com_31, ingredient=ing_29
    )
    com_ing_328 = Component_ingredient.objects.create(
        component=com_31, ingredient=ing_99
    )
    com_ing_329 = Component_ingredient.objects.create(
        component=com_32, ingredient=ing_129
    )
    com_ing_330 = Component_ingredient.objects.create(
        component=com_32, ingredient=ing_9
    )
    com_ing_331 = Component_ingredient.objects.create(
        component=com_32, ingredient=ing_138
    )
    com_ing_332 = Component_ingredient.objects.create(
        component=com_32, ingredient=ing_72
    )
    com_ing_333 = Component_ingredient.objects.create(
        component=com_32, ingredient=ing_150
    )
    com_ing_334 = Component_ingredient.objects.create(
        component=com_32, ingredient=ing_152
    )
    com_ing_335 = Component_ingredient.objects.create(
        component=com_32, ingredient=ing_48
    )
    com_ing_336 = Component_ingredient.objects.create(
        component=com_32, ingredient=ing_31
    )
    com_ing_337 = Component_ingredient.objects.create(
        component=com_32, ingredient=ing_166
    )
    com_ing_338 = Component_ingredient.objects.create(
        component=com_32, ingredient=ing_37
    )
    com_ing_339 = Component_ingredient.objects.create(
        component=com_32, ingredient=ing_169
    )
    com_ing_340 = Component_ingredient.objects.create(
        component=com_33, ingredient=ing_55
    )
    com_ing_341 = Component_ingredient.objects.create(
        component=com_33, ingredient=ing_1
    )
    com_ing_342 = Component_ingredient.objects.create(
        component=com_33, ingredient=ing_131
    )
    com_ing_343 = Component_ingredient.objects.create(
        component=com_33, ingredient=ing_138
    )
    com_ing_344 = Component_ingredient.objects.create(
        component=com_33, ingredient=ing_150
    )
    com_ing_345 = Component_ingredient.objects.create(
        component=com_33, ingredient=ing_152
    )
    com_ing_346 = Component_ingredient.objects.create(
        component=com_33, ingredient=ing_166
    )
    com_ing_347 = Component_ingredient.objects.create(
        component=com_33, ingredient=ing_81
    )
    com_ing_348 = Component_ingredient.objects.create(
        component=com_33, ingredient=ing_61
    )
    com_ing_349 = Component_ingredient.objects.create(
        component=com_33, ingredient=ing_97
    )
    com_ing_350 = Component_ingredient.objects.create(
        component=com_34, ingredient=ing_43
    )
    com_ing_351 = Component_ingredient.objects.create(
        component=com_34, ingredient=ing_128
    )
    com_ing_352 = Component_ingredient.objects.create(
        component=com_34, ingredient=ing_65
    )
    com_ing_353 = Component_ingredient.objects.create(
        component=com_34, ingredient=ing_44
    )
    com_ing_354 = Component_ingredient.objects.create(
        component=com_34, ingredient=ing_45
    )
    com_ing_355 = Component_ingredient.objects.create(
        component=com_34, ingredient=ing_46
    )
    com_ing_356 = Component_ingredient.objects.create(
        component=com_34, ingredient=ing_137
    )
    com_ing_357 = Component_ingredient.objects.create(
        component=com_34, ingredient=ing_138
    )
    com_ing_358 = Component_ingredient.objects.create(
        component=com_34, ingredient=ing_141
    )
    com_ing_359 = Component_ingredient.objects.create(
        component=com_34, ingredient=ing_142
    )
    com_ing_360 = Component_ingredient.objects.create(
        component=com_34, ingredient=ing_47
    )
    com_ing_361 = Component_ingredient.objects.create(
        component=com_34, ingredient=ing_152
    )
    com_ing_362 = Component_ingredient.objects.create(
        component=com_34, ingredient=ing_48
    )
    com_ing_363 = Component_ingredient.objects.create(
        component=com_34, ingredient=ing_49
    )
    com_ing_364 = Component_ingredient.objects.create(
        component=com_34, ingredient=ing_159
    )
    com_ing_365 = Component_ingredient.objects.create(
        component=com_34, ingredient=ing_50
    )
    com_ing_366 = Component_ingredient.objects.create(
        component=com_34, ingredient=ing_163
    )
    com_ing_367 = Component_ingredient.objects.create(
        component=com_34, ingredient=ing_51
    )
    com_ing_368 = Component_ingredient.objects.create(
        component=com_34, ingredient=ing_166
    )
    com_ing_369 = Component_ingredient.objects.create(
        component=com_34, ingredient=ing_169
    )
    com_ing_370 = Component_ingredient.objects.create(
        component=com_35, ingredient=ing_55
    )
    com_ing_371 = Component_ingredient.objects.create(
        component=com_35, ingredient=ing_129
    )
    com_ing_372 = Component_ingredient.objects.create(
        component=com_35, ingredient=ing_45
    )
    com_ing_373 = Component_ingredient.objects.create(
        component=com_35, ingredient=ing_137
    )
    com_ing_374 = Component_ingredient.objects.create(
        component=com_35, ingredient=ing_11
    )
    com_ing_375 = Component_ingredient.objects.create(
        component=com_35, ingredient=ing_138
    )
    com_ing_376 = Component_ingredient.objects.create(
        component=com_35, ingredient=ing_139
    )
    com_ing_377 = Component_ingredient.objects.create(
        component=com_35, ingredient=ing_152
    )
    com_ing_378 = Component_ingredient.objects.create(
        component=com_35, ingredient=ing_92
    )
    com_ing_379 = Component_ingredient.objects.create(
        component=com_35, ingredient=ing_95
    )
    com_ing_380 = Component_ingredient.objects.create(
        component=com_35, ingredient=ing_169
    )
    com_ing_381 = Component_ingredient.objects.create(
        component=com_36, ingredient=ing_4
    )
    com_ing_382 = Component_ingredient.objects.create(
        component=com_36, ingredient=ing_9
    )
    com_ing_383 = Component_ingredient.objects.create(
        component=com_36, ingredient=ing_137
    )
    com_ing_384 = Component_ingredient.objects.create(
        component=com_36, ingredient=ing_138
    )
    com_ing_385 = Component_ingredient.objects.create(
        component=com_36, ingredient=ing_150
    )
    com_ing_386 = Component_ingredient.objects.create(
        component=com_36, ingredient=ing_152
    )
    com_ing_387 = Component_ingredient.objects.create(
        component=com_36, ingredient=ing_48
    )
    com_ing_388 = Component_ingredient.objects.create(
        component=com_36, ingredient=ing_162
    )
    com_ing_389 = Component_ingredient.objects.create(
        component=com_36, ingredient=ing_166
    )
    com_ing_390 = Component_ingredient.objects.create(
        component=com_36, ingredient=ing_33
    )
    com_ing_391 = Component_ingredient.objects.create(
        component=com_36, ingredient=ing_169
    )
    com_ing_392 = Component_ingredient.objects.create(
        component=com_37, ingredient=ing_129
    )
    com_ing_393 = Component_ingredient.objects.create(
        component=com_37, ingredient=ing_65
    )
    com_ing_394 = Component_ingredient.objects.create(
        component=com_37, ingredient=ing_104
    )
    com_ing_395 = Component_ingredient.objects.create(
        component=com_37, ingredient=ing_134
    )
    com_ing_396 = Component_ingredient.objects.create(
        component=com_37, ingredient=ing_108
    )
    com_ing_397 = Component_ingredient.objects.create(
        component=com_37, ingredient=ing_138
    )
    com_ing_398 = Component_ingredient.objects.create(
        component=com_37, ingredient=ing_66
    )
    com_ing_399 = Component_ingredient.objects.create(
        component=com_37, ingredient=ing_152
    )
    com_ing_400 = Component_ingredient.objects.create(
        component=com_37, ingredient=ing_48
    )
    com_ing_401 = Component_ingredient.objects.create(
        component=com_37, ingredient=ing_164
    )
    com_ing_402 = Component_ingredient.objects.create(
        component=com_37, ingredient=ing_166
    )
    com_ing_403 = Component_ingredient.objects.create(
        component=com_37, ingredient=ing_169
    )
    com_ing_404 = Component_ingredient.objects.create(
        component=com_38, ingredient=ing_43
    )
    com_ing_405 = Component_ingredient.objects.create(
        component=com_38, ingredient=ing_4
    )
    com_ing_406 = Component_ingredient.objects.create(
        component=com_38, ingredient=ing_44
    )
    com_ing_407 = Component_ingredient.objects.create(
        component=com_38, ingredient=ing_45
    )
    com_ing_408 = Component_ingredient.objects.create(
        component=com_38, ingredient=ing_46
    )
    com_ing_409 = Component_ingredient.objects.create(
        component=com_38, ingredient=ing_11
    )
    com_ing_410 = Component_ingredient.objects.create(
        component=com_38, ingredient=ing_138
    )
    com_ing_411 = Component_ingredient.objects.create(
        component=com_38, ingredient=ing_47
    )
    com_ing_412 = Component_ingredient.objects.create(
        component=com_38, ingredient=ing_13
    )
    com_ing_413 = Component_ingredient.objects.create(
        component=com_38, ingredient=ing_152
    )
    com_ing_414 = Component_ingredient.objects.create(
        component=com_38, ingredient=ing_48
    )
    com_ing_415 = Component_ingredient.objects.create(
        component=com_38, ingredient=ing_49
    )
    com_ing_416 = Component_ingredient.objects.create(
        component=com_38, ingredient=ing_50
    )
    com_ing_417 = Component_ingredient.objects.create(
        component=com_38, ingredient=ing_162
    )
    com_ing_418 = Component_ingredient.objects.create(
        component=com_38, ingredient=ing_163
    )
    com_ing_419 = Component_ingredient.objects.create(
        component=com_38, ingredient=ing_51
    )
    com_ing_420 = Component_ingredient.objects.create(
        component=com_38, ingredient=ing_61
    )
    com_ing_421 = Component_ingredient.objects.create(
        component=com_39, ingredient=ing_82
    )
    com_ing_422 = Component_ingredient.objects.create(
        component=com_39, ingredient=ing_43
    )
    com_ing_423 = Component_ingredient.objects.create(
        component=com_39, ingredient=ing_55
    )
    com_ing_424 = Component_ingredient.objects.create(
        component=com_39, ingredient=ing_7
    )
    com_ing_425 = Component_ingredient.objects.create(
        component=com_39, ingredient=ing_135
    )
    com_ing_426 = Component_ingredient.objects.create(
        component=com_39, ingredient=ing_138
    )
    com_ing_427 = Component_ingredient.objects.create(
        component=com_39, ingredient=ing_147
    )
    com_ing_428 = Component_ingredient.objects.create(
        component=com_39, ingredient=ing_88
    )
    com_ing_429 = Component_ingredient.objects.create(
        component=com_39, ingredient=ing_152
    )
    com_ing_430 = Component_ingredient.objects.create(
        component=com_39, ingredient=ing_49
    )
    com_ing_431 = Component_ingredient.objects.create(
        component=com_39, ingredient=ing_32
    )
    com_ing_432 = Component_ingredient.objects.create(
        component=com_39, ingredient=ing_63
    )
    #
    print_rows(Component_ingredient.objects.filter(date=None))

    # -----------------------------------------------------------------------------
    ri_1 = Restricted_item.objects.create(
        name="fish", restricted_item_group="fish&seafood", description=""
    )
    ri_2 = Restricted_item.objects.create(
        name="seafood", restricted_item_group="fish&seafood", description=""
    )
    ri_3 = Restricted_item.objects.create(
        name="beef", restricted_item_group="meat", description=""
    )
    ri_4 = Restricted_item.objects.create(
        name="lamb", restricted_item_group="meat", description="gluten, salt"
    )
    ri_5 = Restricted_item.objects.create(
        name="pork", restricted_item_group="meat", description="text"
    )
    ri_6 = Restricted_item.objects.create(
        name="poultry", restricted_item_group="meat", description="text"
    )
    ri_7 = Restricted_item.objects.create(
        name="sausage", restricted_item_group="meat", description="text"
    )
    ri_8 = Restricted_item.objects.create(
        name="turkey ", restricted_item_group="meat", description="text"
    )
    ri_9 = Restricted_item.objects.create(
        name="nuts ", restricted_item_group="nuts&seeds", description=""
    )
    ri_10 = Restricted_item.objects.create(
        name="peanut butter", restricted_item_group="nuts&seeds", description=""
    )
    ri_11 = Restricted_item.objects.create(
        name="peanuts", restricted_item_group="nuts&seeds", description="gluten, salt"
    )
    ri_12 = Restricted_item.objects.create(
        name="seeds", restricted_item_group="nuts&seeds", description="text"
    )
    ri_13 = Restricted_item.objects.create(
        name="dairy", restricted_item_group="other", description="text"
    )
    ri_14 = Restricted_item.objects.create(
        name="gluten", restricted_item_group="other", description="text"
    )
    ri_15 = Restricted_item.objects.create(
        name="spicy", restricted_item_group="other", description="text"
    )
    ri_16 = Restricted_item.objects.create(
        name="green veggies", restricted_item_group="vegetables", description=""
    )
    ri_17 = Restricted_item.objects.create(
        name="leafy greens", restricted_item_group="vegetables", description=""
    )
    ri_18 = Restricted_item.objects.create(
        name="legumes", restricted_item_group="vegetables", description="gluten, salt"
    )
    #
    print_rows(Restricted_item.objects)

    # -----------------------------------------------------------------------------
    inc_1 = Incompatibility.objects.create(restricted_item=ri_3, ingredient=ing_69)
    inc_2 = Incompatibility.objects.create(restricted_item=ri_3, ingredient=ing_72)
    inc_3 = Incompatibility.objects.create(restricted_item=ri_3, ingredient=ing_78)
    inc_4 = Incompatibility.objects.create(restricted_item=ri_13, ingredient=ing_1)
    inc_5 = Incompatibility.objects.create(restricted_item=ri_13, ingredient=ing_2)
    inc_6 = Incompatibility.objects.create(restricted_item=ri_13, ingredient=ing_3)
    inc_7 = Incompatibility.objects.create(restricted_item=ri_13, ingredient=ing_4)
    inc_8 = Incompatibility.objects.create(restricted_item=ri_13, ingredient=ing_5)
    inc_9 = Incompatibility.objects.create(restricted_item=ri_13, ingredient=ing_6)
    inc_10 = Incompatibility.objects.create(restricted_item=ri_13, ingredient=ing_7)
    inc_11 = Incompatibility.objects.create(restricted_item=ri_13, ingredient=ing_8)
    inc_12 = Incompatibility.objects.create(restricted_item=ri_13, ingredient=ing_9)
    inc_13 = Incompatibility.objects.create(restricted_item=ri_13, ingredient=ing_10)
    inc_14 = Incompatibility.objects.create(restricted_item=ri_13, ingredient=ing_13)
    inc_15 = Incompatibility.objects.create(restricted_item=ri_13, ingredient=ing_15)
    inc_16 = Incompatibility.objects.create(restricted_item=ri_1, ingredient=ing_38)
    inc_17 = Incompatibility.objects.create(restricted_item=ri_1, ingredient=ing_39)
    inc_18 = Incompatibility.objects.create(restricted_item=ri_1, ingredient=ing_40)
    inc_19 = Incompatibility.objects.create(restricted_item=ri_1, ingredient=ing_41)
    inc_20 = Incompatibility.objects.create(restricted_item=ri_1, ingredient=ing_42)
    inc_21 = Incompatibility.objects.create(restricted_item=ri_14, ingredient=ing_52)
    inc_22 = Incompatibility.objects.create(restricted_item=ri_14, ingredient=ing_54)
    inc_23 = Incompatibility.objects.create(restricted_item=ri_14, ingredient=ing_57)
    inc_24 = Incompatibility.objects.create(restricted_item=ri_14, ingredient=ing_95)
    inc_25 = Incompatibility.objects.create(restricted_item=ri_14, ingredient=ing_61)
    inc_26 = Incompatibility.objects.create(restricted_item=ri_14, ingredient=ing_62)
    inc_27 = Incompatibility.objects.create(restricted_item=ri_14, ingredient=ing_33)
    inc_28 = Incompatibility.objects.create(restricted_item=ri_14, ingredient=ing_34)
    inc_29 = Incompatibility.objects.create(restricted_item=ri_14, ingredient=ing_35)
    inc_30 = Incompatibility.objects.create(restricted_item=ri_14, ingredient=ing_36)
    inc_31 = Incompatibility.objects.create(restricted_item=ri_14, ingredient=ing_37)
    inc_32 = Incompatibility.objects.create(restricted_item=ri_16, ingredient=ing_121)
    inc_33 = Incompatibility.objects.create(restricted_item=ri_16, ingredient=ing_125)
    inc_34 = Incompatibility.objects.create(restricted_item=ri_16, ingredient=ing_126)
    inc_35 = Incompatibility.objects.create(restricted_item=ri_16, ingredient=ing_127)
    inc_36 = Incompatibility.objects.create(restricted_item=ri_16, ingredient=ing_133)
    inc_37 = Incompatibility.objects.create(restricted_item=ri_16, ingredient=ing_136)
    inc_38 = Incompatibility.objects.create(restricted_item=ri_16, ingredient=ing_140)
    inc_39 = Incompatibility.objects.create(restricted_item=ri_16, ingredient=ing_141)
    inc_40 = Incompatibility.objects.create(restricted_item=ri_16, ingredient=ing_142)
    inc_41 = Incompatibility.objects.create(restricted_item=ri_16, ingredient=ing_143)
    inc_42 = Incompatibility.objects.create(restricted_item=ri_16, ingredient=ing_147)
    inc_43 = Incompatibility.objects.create(restricted_item=ri_16, ingredient=ing_160)
    inc_44 = Incompatibility.objects.create(restricted_item=ri_16, ingredient=ing_161)
    inc_45 = Incompatibility.objects.create(restricted_item=ri_16, ingredient=ing_162)
    inc_46 = Incompatibility.objects.create(restricted_item=ri_16, ingredient=ing_165)
    inc_47 = Incompatibility.objects.create(restricted_item=ri_16, ingredient=ing_169)
    inc_48 = Incompatibility.objects.create(restricted_item=ri_4, ingredient=ing_75)
    inc_49 = Incompatibility.objects.create(restricted_item=ri_17, ingredient=ing_121)
    inc_50 = Incompatibility.objects.create(restricted_item=ri_17, ingredient=ing_133)
    inc_51 = Incompatibility.objects.create(restricted_item=ri_17, ingredient=ing_143)
    inc_52 = Incompatibility.objects.create(restricted_item=ri_17, ingredient=ing_147)
    inc_53 = Incompatibility.objects.create(restricted_item=ri_17, ingredient=ing_162)
    inc_54 = Incompatibility.objects.create(restricted_item=ri_17, ingredient=ing_165)
    inc_55 = Incompatibility.objects.create(restricted_item=ri_18, ingredient=ing_64)
    inc_56 = Incompatibility.objects.create(restricted_item=ri_18, ingredient=ing_65)
    inc_57 = Incompatibility.objects.create(restricted_item=ri_18, ingredient=ing_66)
    inc_58 = Incompatibility.objects.create(restricted_item=ri_18, ingredient=ing_67)
    inc_59 = Incompatibility.objects.create(restricted_item=ri_18, ingredient=ing_14)
    inc_60 = Incompatibility.objects.create(restricted_item=ri_18, ingredient=ing_68)
    inc_61 = Incompatibility.objects.create(restricted_item=ri_9, ingredient=ing_16)
    inc_62 = Incompatibility.objects.create(restricted_item=ri_9, ingredient=ing_17)
    inc_63 = Incompatibility.objects.create(restricted_item=ri_9, ingredient=ing_24)
    inc_64 = Incompatibility.objects.create(restricted_item=ri_9, ingredient=ing_25)
    inc_65 = Incompatibility.objects.create(restricted_item=ri_9, ingredient=ing_32)
    inc_66 = Incompatibility.objects.create(restricted_item=ri_10, ingredient=ing_24)
    inc_67 = Incompatibility.objects.create(restricted_item=ri_11, ingredient=ing_24)
    inc_68 = Incompatibility.objects.create(restricted_item=ri_11, ingredient=ing_25)
    inc_69 = Incompatibility.objects.create(restricted_item=ri_5, ingredient=ing_73)
    inc_70 = Incompatibility.objects.create(restricted_item=ri_5, ingredient=ing_74)
    inc_71 = Incompatibility.objects.create(restricted_item=ri_5, ingredient=ing_77)
    inc_72 = Incompatibility.objects.create(restricted_item=ri_6, ingredient=ing_71)
    inc_73 = Incompatibility.objects.create(restricted_item=ri_6, ingredient=ing_81)
    inc_74 = Incompatibility.objects.create(restricted_item=ri_7, ingredient=ing_77)
    inc_75 = Incompatibility.objects.create(restricted_item=ri_2, ingredient=ing_98)
    inc_76 = Incompatibility.objects.create(restricted_item=ri_2, ingredient=ing_99)
    inc_77 = Incompatibility.objects.create(restricted_item=ri_12, ingredient=ing_30)
    inc_78 = Incompatibility.objects.create(restricted_item=ri_15, ingredient=ing_104)
    inc_79 = Incompatibility.objects.create(restricted_item=ri_15, ingredient=ing_109)
    inc_80 = Incompatibility.objects.create(restricted_item=ri_8, ingredient=ing_81)
    #
    print_rows(Incompatibility.objects)

    # =====================================================
    print("END dataload")



import datetime

from django.contrib.auth.models import User
from django.test import TestCase

from souschef.meal.models import (
    Component,
    Component_ingredient,
    Incompatibility,
    Ingredient,
    Menu,
    Menu_component,
    Restricted_item,
)


class ComponentTestCase(TestCase):
    @classmethod
    def setUpTestData(cls):
        Component.objects.create(
            name="Coq au vin",
            component_group="main dish",
        )

    def test_str_is_fullname(self):
        """Component's string representation is its name"""
        name = "Coq au vin"
        component = Component.objects.get(name=name)
        self.assertEqual(name, str(component))


class Component_ingredientTestCase(TestCase):
    @classmethod
    def setUpTestData(cls):
        component = Component.objects.create(
            name="Coq au vin", component_group="main dish"
        )
        ingredient = Ingredient.objects.create(
            name="Green peas", ingredient_group="veggies and fruits"
        )
        Component_ingredient.objects.create(component=component, ingredient=ingredient)

    def test_str_includes_all_names(self):
        """A Component_ingredient's string representation includes the name
        of the component and the name of the ingredient.
        """
        component = Component.objects.get(name="Coq au vin")
        ingredient = Ingredient.objects.get(name="Green peas")
        component_ingredient = Component_ingredient.objects.get(
            component=component, ingredient=ingredient
        )
        self.assertTrue(component.name in str(component_ingredient))
        self.assertTrue(ingredient.name in str(component_ingredient))


class IncompatibilityTestCase(TestCase):
    @classmethod
    def setUpTestData(cls):
        restricted_item = Restricted_item.objects.create(
            name="pork", restricted_item_group="meat"
        )
        ingredient = Ingredient.objects.create(name="ham", ingredient_group="meat")
        Incompatibility.objects.create(
            restricted_item=restricted_item, ingredient=ingredient
        )

    def test_str_includes_all_names(self):
        """Incompatibility's string representation includes the name
        of the restricted_item and the name of the ingredient.
        """
        restricted_item = Restricted_item.objects.get(name="pork")
        ingredient = Ingredient.objects.get(name="ham")
        incompatibility = Incompatibility.objects.get(
            restricted_item=restricted_item, ingredient=ingredient
        )
        self.assertTrue(restricted_item.name in str(incompatibility))
        self.assertTrue(ingredient.name in str(incompatibility))


class IngredientTestCase(TestCase):
    @classmethod
    def setUpTestData(cls):
        Ingredient.objects.create(
            name="Green peas",
            ingredient_group="veggies and fruits",
        )

    def test_str_is_fullname(self):
        """Ingredient's string representation is its name"""
        name = "Green peas"
        ingredient = Ingredient.objects.get(name=name)
        self.assertEqual(name, str(ingredient))


class MenuTestCase(TestCase):
    @classmethod
    def setUpTestData(cls):
        Menu.objects.create(
            date=datetime.date(2016, 5, 21),
        )

    def test_str_is_fullname(self):
        """Menu's string representation includes date"""
        date = datetime.date(2016, 5, 21)
        menu = Menu.objects.get(date=date)
        self.assertTrue(str(date) in str(menu))


class Menu_componentTestCase(TestCase):
    @classmethod
    def setUpTestData(cls):
        menu = Menu.objects.create(date=datetime.date(2016, 5, 21))
        component = Component.objects.create(
            name="Coq au vin", component_group="main dish"
        )
        Menu_component.objects.create(menu=menu, component=component)

    def test_str_includes_all_names(self):
        """A Menu_component's string representation includes the date
        of the menu and the name of the component.
        """
        date = datetime.date(2016, 5, 21)
        menu = Menu.objects.get(date=date)
        component = Component.objects.get(name="Coq au vin")
        menu_component = Menu_component.objects.get(menu=menu, component=component)
        self.assertTrue(str(date) in str(menu_component))
        self.assertTrue(component.name in str(menu_component))


class CreateMenuAndComponentsTestCase(TestCase):
    fixtures = ["meal_initial_data"]

    def test_menu_new(self):
        number = Menu.create_menu_and_components(
            datetime.date(2016, 7, 15),
            [
                "Ginger pork",
                "Green Salad",
                "Fruit Salad",
                "Day s Dessert",
                "Day s Diabetic Dessert",
                "Day s Pudding",
                "Day s Compote",
            ],
        )
        self.assertEqual(number, 7)

    def test_menu_existed(self):
        Menu.create_menu_and_components(
            datetime.date(2016, 7, 15),
            [
                "Ginger pork",
                "Green Salad",
                "Fruit Salad",
                "Day s Dessert",
                "Day s Diabetic Dessert",
                "Day s Pudding",
                "Day s Compote",
            ],
        )
        number = Menu.create_menu_and_components(
            datetime.date(2016, 7, 15),
            [
                "Coq au vin",
                "Green Salad",
                "Fruit Salad",
                "Day s Dessert",
                "Day s Diabetic Dessert",
                "Day s Pudding",
            ],
        )
        self.assertEqual(number, 6)

    def test_dish_does_not_exist_exception(self):
        with self.assertRaises(Exception) as cm:
            Menu.create_menu_and_components(
                datetime.date(2016, 7, 15),
                [
                    "Chateaubriand Steak",
                    "Green Salad",
                    "Fruit Salad",
                    "Day s Dessert",
                    "Day s Diabetic Dessert",
                    "Day s Pudding",
                    "Day s Compote",
                ],
            )
            the_exception = cm.exception
            self.assertEqual(the_exception.error_code, 3)

    def test_many_dishes_per_component_group_exception(self):
        with self.assertRaises(Exception) as cm:
            Menu.create_menu_and_components(
                datetime.date(2016, 7, 15),
                [
                    "Ginger pork",
                    "Coq au vin",
                    "Green Salad",
                    "Fruit Salad",
                    "Day s Dessert",
                    "Day s Diabetic Dessert",
                    "Day s Pudding",
                    "Day s Compote",
                ],
            )
            the_exception = cm.exception
            self.assertEqual(the_exception.error_code, 3)

    def test_no_main_dish_exception(self):
        with self.assertRaises(Exception) as cm:
            Menu.create_menu_and_components(
                datetime.date(2016, 7, 15),
                [
                    "Green Salad",
                    "Fruit Salad",
                    "Day s Dessert",
                    "Day s Diabetic Dessert",
                    "Day s Pudding",
                    "Day s Compote",
                ],
            )
            the_exception = cm.exception
            self.assertEqual(the_exception.error_code, 3)


class Restricted_itemTestCase(TestCase):
    @classmethod
    def setUpTestData(cls):
        Restricted_item.objects.create(
            name="pork",
            restricted_item_group="meat",
        )

    def test_str_is_fullname(self):
        """Restricted_Item's string representation is its name"""
        name = "pork"
        restricted_item = Restricted_item.objects.get(name=name)
        self.assertEqual(name, str(restricted_item))


class MealAdminTestCase(TestCase):
    fixtures = ["sample_data"]

    @classmethod
    def setUpTestData(cls):
        cls.admin = User.objects.create_superuser(
            username="admin@example.com", email="admin@example.com", password="test1234"
        )

    def setUp(self):
        self.client.login(username=self.admin.username, password="test1234")

    def test_component_search(self):
        response = self.client.get("/admin/meal/component/?q=ess")
        self.assertTrue(b"Diabetic" in response.content)

    def test_ingredient_search(self):
        response = self.client.get("/admin/meal/ingredient/?q=ann")
        self.assertTrue(b"Canned" in response.content)

    def test_restricted_item_search(self):
        response = self.client.get("/admin/meal/restricted_item/?q=ea")
        self.assertTrue(b"eef" in response.content)



import csv
import json
from datetime import datetime

from django.contrib import messages
from django.contrib.auth.mixins import (
    LoginRequiredMixin,
    PermissionRequiredMixin,
)
from django.http import HttpResponse
from django.http.response import HttpResponseRedirect
from django.shortcuts import get_object_or_404
from django.urls import (
    reverse,
    reverse_lazy,
)
from django.utils import timezone
from django.utils.decorators import method_decorator
from django.utils.translation import ugettext_lazy as _
from django.views import (
    View,
    generic,
)
from django.views.decorators.csrf import csrf_exempt
from extra_views import (
    CreateWithInlinesView,
    UpdateWithInlinesView,
)

from souschef.meal.models import (
    COMPONENT_GROUP_CHOICES,
    COMPONENT_GROUP_CHOICES_SIDES,
)
from souschef.meal.settings import COMPONENT_SYSTEM_DEFAULT
from souschef.member.models import (
    DAYS_OF_WEEK,
    Client,
)
from souschef.order.forms import (
    CreateOrderItem,
    CreateOrdersBatchForm,
    OrderStatusChangeForm,
    UpdateOrderItem,
)
from souschef.order.mixins import (
    AjaxableResponseMixin,
    FormValidAjaxableResponseMixin,
)
from souschef.order.models import (
    ORDER_STATUS,
    ORDER_STATUS_CANCELLED,
    Order,
    OrderFilter,
    OrderStatusChange,
)


class OrderList(LoginRequiredMixin, PermissionRequiredMixin, generic.ListView):
    context_object_name = "orders"
    model = Order
    paginate_by = 20
    permission_required = "sous_chef.read"
    template_name = "list.html"

    def get_queryset(self):
        uf = OrderFilter(self.request.GET)
        return uf.qs.select_related("client__member").prefetch_related("orders")

    def get_context_data(self, **kwargs):
        uf = OrderFilter(self.request.GET, queryset=self.get_queryset())

        context = super().get_context_data(**kwargs)

        context["filter"] = uf

        text = ""
        count = 0
        for getVariable in self.request.GET:
            if getVariable == "page":
                continue
            for getValue in self.request.GET.getlist(getVariable):
                if count == 0:
                    text += "?" + getVariable + "=" + getValue
                else:
                    text += "&" + getVariable + "=" + getValue
                count += 1

        text = text + "?" if count == 0 else text + "&"
        context["get"] = text

        return context

    def get(self, request, **kwargs):
        self.format = request.GET.get("format", False)

        if self.format == "csv":
            return ExportCSV(self, self.get_queryset())

        return super().get(request, **kwargs)


class OrderDetail(LoginRequiredMixin, PermissionRequiredMixin, generic.DetailView):
    model = Order
    permission_required = "sous_chef.read"
    template_name = "view.html"
    context_object_name = "order"
    queryset = Order.objects.all().prefetch_related("orders")

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context["status"] = ORDER_STATUS
        return context


class CreateOrder(
    AjaxableResponseMixin,
    LoginRequiredMixin,
    PermissionRequiredMixin,
    CreateWithInlinesView,
):
    fields = ["client", "delivery_date"]
    inlines = [CreateOrderItem]
    model = Order
    permission_required = "sous_chef.edit"
    template_name = "create.html"

    # TODO: Change the validation of the form,
    # If component is present, then component_group is not required
    # and vice versa

    def get_success_url(self):
        return self.object.get_absolute_url()


class CreateOrdersBatch(LoginRequiredMixin, PermissionRequiredMixin, generic.FormView):
    permission_required = "sous_chef.edit"
    success_url = reverse_lazy("order:create_batch")
    template_name = "order/create_batch.html"

    def get_form(self, *args, **kwargs):
        k = self.get_form_kwargs()  # kwargs to initialize the form
        if self.request.method == "POST" and self.request.POST.get("delivery_dates"):
            k["delivery_dates"] = self.request.POST["delivery_dates"].split("|")

        return CreateOrdersBatchForm(**k)

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        # Define here any needed variable for template
        context["meals"] = list(
            filter(
                lambda tup: tup[0] != COMPONENT_GROUP_CHOICES_SIDES,
                COMPONENT_GROUP_CHOICES,
            )
        )

        # delivery_dates
        if self.request.method == "POST" and self.request.POST.get("delivery_dates"):
            delivery_dates = self.request.POST["delivery_dates"].split("|")
        else:
            delivery_dates = []

        # dates of orders to override
        if self.request.method == "POST" and self.request.POST.get("override_dates"):
            override_dates = self.request.POST["override_dates"].split("|")
            override_dates = [x for x in override_dates if x in delivery_dates]
        else:
            override_dates = []

        # inactive accordion dates
        if self.request.method == "POST" and self.request.POST.get(
            "accordions_inactive"
        ):
            context["accordions_inactive"] = self.request.POST[
                "accordions_inactive"
            ].split("|")
        else:
            context["accordions_inactive"] = []

        if self.request.method == "POST" and self.request.POST.get("client"):
            c = Client.objects.get(pk=self.request.POST["client"])
            meals_default_dict = dict(c.meals_default)
            context["client"] = c

            # The dates where an order already exists.
            today = timezone.datetime.today()
            ordered_dates = (
                c.orders.filter(delivery_date__gte=today)
                .exclude(status="C")
                .order_by("delivery_date")
                .values_list("delivery_date", flat=True)
            )
            context["ordered_dates"] = "|".join(
                map(lambda d: d.strftime("%Y-%m-%d"), ordered_dates)
            )
        else:
            meals_default_dict = dict(
                map(lambda x: (x[0], None), DAYS_OF_WEEK),
            )

        context["delivery_dates"] = []
        context["override_orders"] = []
        for date in delivery_dates:
            date_obj = datetime.strptime(date, "%Y-%m-%d")
            # sunday = 0, saturday = 6
            day = (
                "sunday",
                "monday",
                "tuesday",
                "wednesday",
                "thursday",
                "friday",
                "saturday",
            )[int(date_obj.strftime("%w"))]

            if date not in override_dates and context.get("client"):
                order_on_day = (
                    context["client"]
                    .orders.filter(delivery_date=date_obj)
                    .exclude(status="C")
                    .first()
                )
                if context.get("client") and order_on_day:
                    # the client has an active order already on this day.
                    # show the warning modal if it's not already being shown.
                    context["show_override_modal"] = True
                    context["override_orders"].append(order_on_day)

            if not meals_default_dict[day]:  # None or {}
                # system default
                default_json = json.dumps(COMPONENT_SYSTEM_DEFAULT)
            else:
                # client default
                default_json = json.dumps(meals_default_dict[day])

            context["delivery_dates"].append((date, date_obj, default_json))

        context["override_dates"] = "|".join(override_dates)
        return context

    def form_invalid(self, form, **kwargs):
        # open accordions if there's an invalid field in it.
        context = self.get_context_data(**kwargs)
        context["form"] = form
        if form.cleaned_data.get("is_submit") != 1:
            form._errors = {}
        else:
            for field in form.errors:
                try:
                    # next() - find first occurence
                    invalid_date = next(
                        date for date in context["accordions_inactive"] if date in field
                    )
                    context["accordions_inactive"].remove(invalid_date)
                except StopIteration:
                    pass

        return self.render_to_response(context)

    def form_valid(self, form):
        # Get posted datas
        del_dates = form.cleaned_data["delivery_dates"].split("|")
        ovr_dates = form.cleaned_data["override_dates"].split("|")
        client = form.cleaned_data["client"]
        items = form.cleaned_data
        del items["delivery_dates"]
        del items["client"]

        # Place orders using posted datas
        created_orders = Order.objects.create_batch_orders(
            del_dates, client, items, ovr_dates
        )

        # check created and uncreated dates
        created_dates = []
        for order in created_orders:
            d = order.delivery_date.strftime("%Y-%m-%d")
            created_dates.append(d)
        uncreated_dates = []
        for d in del_dates:
            if d not in created_dates:
                uncreated_dates.append(d)

        # Alert user on order placement
        if created_dates:
            messages.add_message(
                self.request,
                messages.SUCCESS,
                (
                    _(
                        "%(n)s order(s) successfully placed "
                        "for %(client)s on %(dates)s."
                    )
                    % {
                        "n": len(created_dates),
                        "client": client,
                        "dates": ", ".join(created_dates),
                    }
                ),
            )
        if uncreated_dates:
            messages.add_message(
                self.request,
                messages.WARNING,
                (
                    _("%(n)s existing order(s) skipped " "for %(client)s on %(dates)s.")
                    % {
                        "n": len(uncreated_dates),
                        "client": client,
                        "dates": ", ".join(uncreated_dates),
                    }
                ),
            )

        response = super().form_valid(form)
        return response


class UpdateOrder(
    LoginRequiredMixin,
    PermissionRequiredMixin,
    AjaxableResponseMixin,
    UpdateWithInlinesView,
):
    fields = ["client", "delivery_date"]
    inlines = [UpdateOrderItem]
    model = Order
    permission_required = "sous_chef.edit"
    template_name = "update.html"

    def get_form(self, *args, **kwargs):
        form = super().get_form(*args, **kwargs)
        form.fields["client"].queryset = (
            Client.objects.all()
            .select_related("member")
            .only("member__firstname", "member__lastname")
        )
        return form

    def get_success_url(self):
        return self.object.get_absolute_url()


class UpdateOrderStatus(
    LoginRequiredMixin,
    PermissionRequiredMixin,
    FormValidAjaxableResponseMixin,
    generic.CreateView,
):
    form_class = OrderStatusChangeForm
    model = OrderStatusChange
    permission_required = "sous_chef.edit"
    template_name = "order_update_status.html"

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context["order"] = get_object_or_404(Order, pk=self.kwargs.get("pk"))
        context["order_status"] = ORDER_STATUS
        context["order_no_charge"] = "N"
        context["order_no_charge_reasons"] = (
            _("Restrictions mistake"),
            _("Delivery mistake"),
            _("Really unhappy client"),
        )
        return context

    def get_initial(self):
        order = get_object_or_404(Order, pk=self.kwargs.get("pk"))
        return {
            "order": self.kwargs.get("pk"),
            "status_from": order.status,
            "status_to": self.request.GET.get("status"),
        }

    def form_valid(self, form):
        response = super().form_valid(form)
        messages.add_message(
            self.request,
            messages.SUCCESS,
            _("The order status was updated successfully."),
        )
        return response

    def get_success_url(self):
        return reverse("order:view", kwargs={"pk": self.kwargs.get("pk")})


@method_decorator(csrf_exempt, name="dispatch")
class CreateDeleteOrderClientBill(LoginRequiredMixin, PermissionRequiredMixin, View):
    permission_required = "sous_chef.edit"

    def get_object(self, pk):
        return get_object_or_404(Order.objects.prefetch_related("orders"), pk=pk)

    def post(self, request, pk, *args, **kwargs):
        order = self.get_object(pk)
        order.includes_a_bill = True
        return HttpResponse("OK", status=200)

    def delete(self, request, pk, *args, **kwargs):
        order = self.get_object(pk)
        order.includes_a_bill = False
        return HttpResponse("OK", status=200)


class CancelOrder(LoginRequiredMixin, PermissionRequiredMixin, View):
    permission_required = "sous_chef.edit"

    def post(self, request, pk, *args, **kwargs):
        order = get_object_or_404(Order, pk=pk)
        order.status = ORDER_STATUS_CANCELLED
        order.save()
        return HttpResponseRedirect(request.GET["next"])


class DeleteOrder(LoginRequiredMixin, PermissionRequiredMixin, generic.DeleteView):
    model = Order
    permission_required = "sous_chef.edit"
    template_name = "order_confirm_delete.html"

    def get_success_url(self):
        # 'next' parameter should always be included in POST'ed URL.
        return self.request.GET.get("next") or self.request.POST["next"]


def ExportCSV(request, queryset):
    response = HttpResponse(content_type="text/csv")
    response["Content-Disposition"] = "attachment; filename=order_export.csv"
    writer = csv.writer(response, csv.excel)

    writer.writerow(
        [
            "ID",
            "Client Firstname",
            "Client Lastname",
            "Order Status",
            "Creation Date",
            "Delivery Date",
            "price",
        ]
    )

    for obj in queryset:
        writer.writerow(
            [
                obj.id,
                obj.client.member.firstname,
                obj.client.member.lastname,
                obj.get_status_display(),
                obj.creation_date,
                obj.delivery_date,
                obj.price,
            ]
        )

    return response



# Generated by Django 1.10.6 on 2017-04-19 15:09

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("note", "0010_auto_20170313_1442"),
    ]

    operations = [
        migrations.AlterField(
            model_name="note",
            name="category",
            field=models.ForeignKey(
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                related_name="notes",
                to="note.NoteCategory",
                verbose_name="Category",
            ),
        ),
        migrations.AlterField(
            model_name="note",
            name="priority",
            field=models.ForeignKey(
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                related_name="notes",
                to="note.NotePriority",
                verbose_name="Priority",
            ),
        ),
    ]



import collections
import re
from datetime import (
    date,
    datetime,
)

from django.core.exceptions import ValidationError
from django.db import (
    connection,
    models,
    transaction,
)
from django.db.models import Q
from django.urls import reverse
from django.utils.translation import ugettext_lazy as _
from django_filters import (
    CharFilter,
    ChoiceFilter,
    FilterSet,
)

from souschef.meal.models import (
    COMPONENT_GROUP_CHOICES,
    COMPONENT_GROUP_CHOICES_MAIN_DISH,
    COMPONENT_GROUP_CHOICES_SIDES,
)
from souschef.member.models import (
    DAYS_OF_WEEK,
    OPTION_GROUP_CHOICES_PREPARATION,
    RATE_TYPE_LOW_INCOME,
    RATE_TYPE_SOLIDARY,
)

ORDER_STATUS = (
    ("O", _("Ordered")),
    ("D", _("Delivered")),
    ("N", _("No Charge")),
    ("C", _("Cancelled")),
    ("B", _("Billed")),
    ("P", _("Paid")),
)

ORDER_STATUS_ORDERED = ORDER_STATUS[0][0]
ORDER_STATUS_DELIVERED = ORDER_STATUS[1][0]
ORDER_STATUS_CANCELLED = ORDER_STATUS[3][0]

SIZE_CHOICES = (
    ("", ""),
    ("R", _("Regular")),
    ("L", _("Large")),
)

SIZE_CHOICES_REGULAR = SIZE_CHOICES[1][0]
SIZE_CHOICES_LARGE = SIZE_CHOICES[2][0]

ORDER_ITEM_TYPE_CHOICES = (
    ("meal_component", _("Meal component (main dish, vegetable, side dish, seasonal)")),
    ("delivery", _("Delivery (general store item, invitation, ...)")),
    ("pickup", _("Pickup (payment)")),
    ("visit", _("Visit")),
)

ORDER_ITEM_TYPE_CHOICES_COMPONENT = ORDER_ITEM_TYPE_CHOICES[0][0]

MAIN_PRICE_DEFAULT = 6.00  # TODO use decimal ?
SIDE_PRICE_DEFAULT = 1.00
MAIN_PRICE_LOW_INCOME = 4.50
SIDE_PRICE_LOW_INCOME = 0.75
MAIN_PRICE_SOLIDARY = 3.50
SIDE_PRICE_SOLIDARY = 0.50


class OrderManager(models.Manager):
    def get_orders(self, delivery_date=None, order_statuses=None):
        # If no date is passed, use the current day
        if delivery_date is None:
            delivery_date = date.today()
        extra_kwargs = {}
        if order_statuses:
            extra_kwargs["status__in"] = order_statuses
        return self.get_queryset().filter(
            delivery_date=delivery_date,
            **extra_kwargs,
        )

    def get_shippable_orders(self, delivery_date=None, exclude_non_geolocalized=False):
        """
        Return the orders ready to be delivered for a given date.
        A shippable order must be created in the database, and its ORDER_STATUS
        must be 'O' (Ordered).
        """
        # If no date is passed, use the current day
        if delivery_date is None:
            delivery_date = date.today()
        extra_kwargs = {}
        if exclude_non_geolocalized is True:
            extra_kwargs["client__member__address__latitude__isnull"] = False
            extra_kwargs["client__member__address__longitude__isnull"] = False
        return self.get_queryset().filter(
            delivery_date=delivery_date, status=ORDER_STATUS_ORDERED, **extra_kwargs
        )

    def get_shippable_orders_by_route(
        self, route_id, delivery_date, exclude_non_geolocalized=False
    ):
        """
        Return the orders ready to be delivered for a given route.
        If delivery date is not provided, it is assumed to today.
        A shippable order must be created in the database, and its ORDER_STATUS
        must be 'O' (Ordered).
        """
        if delivery_date is None:
            delivery_date = date.today()
        extra_kwargs = {}
        if exclude_non_geolocalized is True:
            extra_kwargs["client__member__address__latitude__isnull"] = False
            extra_kwargs["client__member__address__longitude__isnull"] = False
        return self.get_queryset().filter(
            delivery_date=delivery_date,
            status=ORDER_STATUS_ORDERED,
            client__route=route_id,
            **extra_kwargs,
        )

    def get_billable_orders(self, year, month):
        """
        Return the orders that have successfully delivered during
        the given period.
        A period is represented by a year and a month.
        """
        return self.get_queryset().filter(
            delivery_date__year=year,
            delivery_date__month=month,
            status="D",
        )

    def get_billable_orders_client(self, month, year, client):
        """
        Return the orders that have successfully delivered during
        the given period to a given client.
        A period is represented by a year and a month.
        """
        return self.get_queryset().filter(
            delivery_date__year=year,
            delivery_date__month=month,
            client=client,
            status="D",
        )

    def get_client_prices(self, client):
        # TODO Use Parameters Model in member to store unit prices
        if client.rate_type == RATE_TYPE_LOW_INCOME:
            main_price = MAIN_PRICE_LOW_INCOME
            side_price = SIDE_PRICE_LOW_INCOME
        elif client.rate_type == RATE_TYPE_SOLIDARY:
            main_price = MAIN_PRICE_SOLIDARY
            side_price = SIDE_PRICE_SOLIDARY
        else:
            main_price = MAIN_PRICE_DEFAULT
            side_price = SIDE_PRICE_DEFAULT

        return {"main": main_price, "side": side_price}

    def auto_create_orders(self, delivery_date, clients):
        """
        Automatically creates orders and order items for the given delivery
        date and given client list.
        Order items will be created based on client's meals schedule.

        Parameters:
          delivery_date: date on which orders are to be delivered
          clients: a list of one or many client objects

        Returns:
          Created orders.
        """
        created_orders = []
        weekday = delivery_date.weekday()  # Monday is 0, Sunday is 6
        day = DAYS_OF_WEEK[weekday][0]  # 0 -> 'monday', 6 -> 'sunday'
        for client in clients:
            try:
                order = Order.objects.get(client=client, delivery_date=delivery_date)
                created_orders.append(order)
                continue
            except Order.DoesNotExist:
                # If no order for this client/date, create it and attach items
                items = dict(client.meals_schedule).get(day, None)

                # Skip this client if no scheduled delivery
                if items is None:
                    continue

                filtered_items = {k: v for k, v in items.items() if v is not None}

                # Skip this client if nothing to order
                if not filtered_items:
                    continue

                individual_items = {}
                for key, value in filtered_items.items():
                    if "size" in key:
                        replaced_key = key + "_default"
                    else:
                        replaced_key = key + "_default_quantity"
                    individual_items[replaced_key] = value

                prices = self.get_client_prices(client)
                order = self.create_order(
                    delivery_date, client, individual_items, prices
                )

                created_orders.append(order)
        return created_orders

    def create_batch_orders(self, delivery_dates, client, items, override_dates):
        """
        Create orders for one or multiple days, for a given client.
        Order items will be created based on client's meals schedule.

        Parameters:
          delivery_dates : dates on which orders are to be created
          client : a client object
          items: the order items
          override_dates: dates on which existing orders can be overriden

        Returns:
          An array of created orders.
        """
        created_orders = []

        # Calculate client prices (main and side)
        prices = self.get_client_prices(client)

        for delivery_date_str in delivery_dates:
            delivery_date = datetime.strptime(delivery_date_str, "%Y-%m-%d").date()
            prior_order = Order.objects.filter(
                client=client, delivery_date=delivery_date
            )
            if prior_order.exists():
                if delivery_date_str in override_dates:
                    # If an order is already created, override the original(s)
                    for x in prior_order:
                        x.status = "C"
                        x.save()
                else:
                    continue
            individual_items = {}
            for key, value in items.items():
                if delivery_date_str in key:
                    replaced_key = key.replace(delivery_date_str, "default")
                    individual_items[replaced_key] = value
            order = self.create_order(delivery_date, client, individual_items, prices)
            created_orders.append(order)

        return created_orders

    @transaction.atomic
    def create_order(self, delivery_date, client, items, prices):
        """
        Create an order for given date, client, items and prices.
        items should be formatted as (example):
            {
                'main_dish_default_quantity': 1,
                'size_default': 'R',
                'dessert_default_quantity': 1,
                'compost_default_quantity': 1,
                'delivery_default': True
                ...
            }
        Every main dish comes with a free side dish. (thus not billable)
        """
        order = Order.objects.create(client=client, delivery_date=delivery_date)
        free_side_dishes = items.get("main_dish_default_quantity") or 0

        for component_group, _trans in COMPONENT_GROUP_CHOICES:
            if component_group != COMPONENT_GROUP_CHOICES_SIDES:
                item_qty = items.get(component_group + "_default_quantity") or 0
                if item_qty == 0:
                    continue

                common_kwargs = {
                    "order": order,
                    "component_group": component_group,
                    "order_item_type": ORDER_ITEM_TYPE_CHOICES_COMPONENT,
                }

                if component_group == COMPONENT_GROUP_CHOICES_MAIN_DISH:
                    price = item_qty * prices["main"]
                    if items["size_default"] == "L":
                        price += item_qty * prices["side"]
                    # main dish
                    Order_item.objects.create(
                        size=items["size_default"],
                        total_quantity=item_qty,
                        price=price,
                        billable_flag=True,
                        **common_kwargs,
                    )
                else:
                    # side dish: deduct+billable
                    deduct = min(free_side_dishes, item_qty)
                    free_side_dishes -= deduct
                    if deduct > 0:
                        # free side dishes
                        Order_item.objects.create(
                            size=None,
                            total_quantity=deduct,
                            price=deduct * prices["side"],
                            billable_flag=False,
                            **common_kwargs,
                        )

                    billable = item_qty - deduct
                    if billable > 0:
                        # billable side dishes
                        Order_item.objects.create(
                            size=None,
                            total_quantity=billable,
                            price=billable * prices["side"],
                            billable_flag=True,
                            **common_kwargs,
                        )

        for order_item_type, _trans in ORDER_ITEM_TYPE_CHOICES:
            if order_item_type != ORDER_ITEM_TYPE_CHOICES_COMPONENT:
                additional = items.get(f"{order_item_type}_default")
                if additional:
                    Order_item.objects.create(
                        order=order,
                        price=0,
                        total_quantity=0,
                        billable_flag=False,
                        order_item_type=order_item_type,
                    )

        return order

    """
    Allow changing status of multiple orders at once.
    """

    def update_orders_status(self, orders, new):
        count = orders.update(status=new)
        return count


class Order(models.Model):
    class Meta:
        verbose_name_plural = _("orders")
        ordering = ["-delivery_date"]

    # Order information
    creation_date = models.DateField(verbose_name=_("creation date"), auto_now_add=True)

    delivery_date = models.DateField(verbose_name=_("delivery date"))

    status = models.CharField(
        max_length=1,
        choices=ORDER_STATUS,
        verbose_name=_("order status"),
        default=ORDER_STATUS_ORDERED,
    )

    client = models.ForeignKey(
        "member.Client",
        verbose_name=_("client"),
        related_name="client_order",
        blank=False,
        default="",
        on_delete=models.CASCADE,
    )

    objects = OrderManager()

    def get_absolute_url(self):
        return reverse("order:view", kwargs={"pk": self.pk})

    @property
    def price(self):
        """
        Sum the items prices for the given order.
        """
        total = 0
        for item in self.orders.all():
            if item.billable_flag is True:
                total = total + item.price
        return total

    @property
    def simple_summary(self):
        """
        Returns a simple summary of the accompanying order items.
        """
        return ", ".join(
            [
                f"{x.total_quantity}x {x.get_component_group_display()}"
                for x in self.orders.all()
                if x.order_item_type == "meal_component" or x.component_group
            ]
        )

    def __str__(self):
        return f"client={self.client}, delivery_date={self.delivery_date}"

    @staticmethod
    def get_kitchen_items(delivery_date):
        """Get all client meal order specifics for delivery date.

        For each client that has ordered a meal for 'delivery_date',
        find all the information needed for the Kitchen Count Report
        and the Meal Labels. This information is stored in KitchenItem
        objects. (See KitchenItem for description of each attribute).

        Args:
            delivery_date: A datetime.date object, the date on which
                the meals will be delivered to the clients.

        Returns:
            A dictionary where the key is an Integer 'client id' and
            the value is a KitchenItem named tuple.
        """
        kitchen_list = {}

        # Day's avoid ingredients clashes.
        for row in day_avoid_ingredient(delivery_date):
            check_for_new_client(kitchen_list, row)
            if (
                row.oiorderid
                and row.menucompid
                and row.component_group == COMPONENT_GROUP_CHOICES_MAIN_DISH
            ):
                # found avoid ingredient clash in main dish
                # should be a sorted set (or ordered dict ?)
                kitchen_list[row.cid].incompatible_ingredients.append(row.ingredient)
            # we Know that it is not incompatible with main dish
            if (
                row.oiorderid
                and row.menucompid
                and row.component_group == COMPONENT_GROUP_CHOICES_SIDES
                and row.ingredient not in kitchen_list[row.cid].sides_clashes
            ):
                # found new avoid ingredient clash in sides
                kitchen_list[row.cid].sides_clashes.append(row.ingredient)

                # This might be reactivated shortly:
                # # we also add side clash ingredients into main ingredient clashes
                # # so they can be grouped and displayed together
                # kitchen_list[row.cid].incompatible_ingredients.append(
                #     f"{row.ingredient} (sides)"
                # )
            if row.ingredient not in kitchen_list[row.cid].avoid_ingredients:
                # remember ingredient to avoid
                kitchen_list[row.cid].avoid_ingredients.append(row.ingredient)

        # Day's restrictions.
        for row in day_restrictions(delivery_date):
            check_for_new_client(kitchen_list, row)
            if (
                row.oiorderid
                and row.menucompid
                and row.component_group == COMPONENT_GROUP_CHOICES_MAIN_DISH
            ):
                # found restriction clash in main dish
                # should be a sorted set
                kitchen_list[row.cid].incompatible_ingredients.append(row.ingredient)
            if (
                row.oiorderid
                and row.menucompid
                and row.component_group == COMPONENT_GROUP_CHOICES_SIDES
                and row.restricted_item not in kitchen_list[row.cid].sides_clashes
            ):
                # found new restriction clash in sides
                kitchen_list[row.cid].sides_clashes.append(row.restricted_item)
            if row.restricted_item not in kitchen_list[row.cid].restricted_items:
                # remember restricted_item
                kitchen_list[row.cid].restricted_items.append(row.restricted_item)

        # Day's preparations.
        for row in day_preparations(delivery_date):
            check_for_new_client(kitchen_list, row)
            # should be a sorted set
            # found client with food preparation
            kitchen_list[row.cid].preparation.append(row.food_prep)

        # Day's Delivery Items, Components summary and Data for all labels.
        for row in day_delivery_items(delivery_date):
            check_for_new_client(kitchen_list, row)
            if row.component_group == COMPONENT_GROUP_CHOICES_MAIN_DISH:
                kitchen_list[row.cid] = kitchen_list[row.cid]._replace(
                    meal_qty=(kitchen_list[row.cid].meal_qty + row.total_quantity),
                    meal_size=row.size,
                )
            old_component = kitchen_list[row.cid].meal_components.get(
                row.component_group
            )
            if old_component:
                # component group already exists in the order
                kitchen_list[row.cid].meal_components[row.component_group] = (
                    old_component._replace(
                        qty=old_component.qty + (row.total_quantity or 0)
                    )
                )
            else:
                # new component group for this order
                kitchen_list[row.cid].meal_components[row.component_group] = (
                    MealComponent(
                        id=row.component_id,
                        name=row.component_name,
                        qty=row.total_quantity or 0,
                    )
                )
            kitchen_list[row.cid] = kitchen_list[row.cid]._replace(
                routename=row.routename
            )

        # Sort requirements list in each value.
        for value in kitchen_list.values():
            value.incompatible_ingredients.sort()
            value.restricted_items.sort()
            value.preparation.sort()

        return kitchen_list

    @staticmethod
    def get_delivery_list(delivery_date, route_id):
        """
        Get all delivery order specifics for delivery date and route.
        Exclude non-geolocalized clients.

        For each client that has ordered a meal for 'delivery_date'
        and that belongs to the route specified, find all the
        information needed to generate the Route Sheet. This
        information is stored in DeliveryClient objects. (See
        DeliveryClient for the attributes).

        Args:
            delivery_date: A datetime.date object, the date on which
                the meals will be delivered to the clients.
            route_id: An integer, the is of the route for which the
                delivery list must be produced.

        Returns:
            A dictionary where the key is an Integer 'client id' and
            the value is a DeliveryClient object.
        """
        orditms = (
            Order_item.objects.select_related("order__client__member__address")
            .exclude(
                order__status=ORDER_STATUS_CANCELLED,
            )
            .filter(
                order__delivery_date=delivery_date, order__client__route__id=route_id
            )
            .order_by("order__client_id")
        )

        route_list = {}
        for oi in orditms:
            if oi.order.client.is_geolocalized is False:
                # exclude non-geolocalized client
                continue
            if not route_list.get(oi.order.client.id):
                # found new client
                route_list[oi.order.client.id] = DeliveryClient(
                    oi.order.client.member.firstname,
                    oi.order.client.member.lastname,
                    oi.order.client.member.address.number,
                    oi.order.client.member.address.street,
                    oi.order.client.member.address.apartment,
                    (
                        oi.order.client.member.home_phone
                        or oi.order.client.member.cell_phone
                    ),
                    oi.order.client.delivery_note,
                    delivery_items=[],
                    order_id=oi.order.id,
                    include_a_bill=oi.order.includes_a_bill,
                )
            # found new delivery item for client
            if (
                oi.order_item_type == ORDER_ITEM_TYPE_CHOICES_COMPONENT
                and oi.component_group
            ):
                # found a meal_component with proper component_group
                for j in range(len(route_list[oi.order.client.id].delivery_items)):
                    if (
                        route_list[oi.order.client.id].delivery_items[j].component_group
                        == oi.component_group
                    ):
                        # existing component_group in this order
                        old_quantity = (
                            route_list[oi.order.client.id]
                            .delivery_items[j]
                            .total_quantity
                        )
                        old_remark = (
                            route_list[oi.order.client.id].delivery_items[j].remark
                        )
                        if old_remark != "":
                            old_remark = old_remark + "; "
                        route_list[oi.order.client.id].delivery_items[j] = (
                            route_list[oi.order.client.id]
                            .delivery_items[j]
                            ._replace(
                                # cumulate quantities
                                total_quantity=(
                                    old_quantity + (oi.total_quantity or 0)
                                ),
                                # concatenate order item remarks
                                remark=(old_remark + (oi.remark or "")),
                            )
                        )
                        break
                else:
                    # new component_group in this order
                    route_list[oi.order.client.id].delivery_items.append(
                        DeliveryItem(
                            oi.component_group,
                            # find the translated name of the component group
                            next(
                                cg
                                for cg in COMPONENT_GROUP_CHOICES
                                if cg[0] == oi.component_group
                            )[1],
                            oi.total_quantity or 0,
                            oi.order_item_type,
                            oi.remark or "",
                            size=(
                                oi.size
                                if oi.component_group
                                == (COMPONENT_GROUP_CHOICES_MAIN_DISH)
                                else ""
                            ),
                        )
                    )

        # Sort delivery items for each client
        for client in route_list.values():
            client.delivery_items.sort(key=component_group_sorting)

        return route_list

    @property
    def includes_a_bill(self):
        return any(item.is_a_client_bill for item in self.orders.all())

    @includes_a_bill.setter
    def includes_a_bill(self, value):
        if value is True:
            if self.includes_a_bill is False:
                Order_item.objects.create(
                    order=self,
                    price=0,
                    billable_flag=False,
                    size=None,
                    order_item_type="delivery",
                    remark=None,
                    total_quantity=0,
                    component_group=None,
                )
        elif value is False:
            if self.includes_a_bill is True:
                for item in self.orders.all():
                    if item.is_a_client_bill:
                        item.delete()
        else:
            raise ValueError("Order.includes_a_bill only accepts " "boolean values.")

    @property
    def client_planned_status_at_delivery(self):
        return self.client.get_status_planned_at_date(self.delivery_date)

    @property
    def client_planned_status_at_delivery_verbose(self):
        return self.client.get_verbose_status(self.client_planned_status_at_delivery)

    @property
    def is_in_error_for_the_kitchen_count(self):
        """Return True if the order is in error for the Kitchen Count
        orders list page.

        This is based on the client, which either:
        - is not geolocalized;
        - has no route;
        - is not active at the delivery date.
        """
        return (
            not self.client.is_geolocalized
            or not self.client.route
            or self.client_planned_status_at_delivery != "A"
        )


# Order.get_kitchen_items helper functions.


def named_tuple_fetchall(cursor):
    """Fetch all rows from a database 'cursor', returning namedtuples.

    Args:
        cursor: A database cursor object on which a query was executed.

    Returns:
        A list of named tuples, each one being a row fetched from the cursor.
        Each column of the cursor becomes an attribute of the named tuple.
    """
    desc = cursor.description
    nt_row = collections.namedtuple("Row", [col[0] for col in desc])
    return [nt_row(*row) for row in cursor.fetchall()]


def sql_prep(query, valuesdict):
    """Prepare SQL 'query' by matching its parameters with given 'values'.

    Find parameters %(name)s in 'query' and replace them with a %s placeholder
    then build an ordered list of corresponding values.

    Args:
        query: A string, the SQL query containing parameters like %(name)s.
        valuesdict: A dictionary of parameter values : {'name': value, ...}.

    Returns:
        A tuple of two values, (the modified query string, a list of values to
        pass to the query). For example :

        >>> sql_prep('this is %(one)s a trial of %(two)s changes %(one)s end',
                     {'one': 10, 'two': 'twenty'})
        ('this is %s a trial of %s changes %s end', [10, 'twenty', 10])

    Raises:
        Exception: A parameter in the query does not have a matching value.
    """
    # Get all the substrings that match the regex
    subs = [(m.start(), m.end(), m.group(0)) for m in re.finditer(r"%\(\w+\)s", query)]
    # print("subs", subs)  # DEBUG
    mod = []  # list of segments of new query
    pos = 0  # start of non-matching text in old query
    names = []  # parameter names found in old query
    for sta, end, grp in subs:
        mod.append(query[pos:sta])  # add text before match
        mod.append("%s")  # new placeholder
        pos = end  # next non-matching text
        names.append(grp[2:-2])  # extract name in %(name)s
    mod.append(query[pos:])  # add tail of old query
    values = []
    for n in names:
        val = valuesdict.get(n)
        if val:
            values.append(val)
        else:
            raise Exception(f"Query parameter '{n}' not found in values")
    return "".join(mod), values


def sql_exec(query, values, heading=""):
    """Execute SQL 'query' passing to it the given 'values'.

    Args:
        query: A string, the SQL query containing parameters like %(name)s.
        values: A dictionary of parameter values : {'name': value, ...}.
        heading: A string that if we debug, will be printed once before
            all the result rows.

    Returns:
        A list of named tuples, each one is a row returned by the database
        as the result of the SQL query.
    """
    with connection.cursor() as cursor:
        cursor.execute(*sql_prep(query, values))
        rows = named_tuple_fetchall(cursor)
    # UNCOMMENT THESE LINES FOR DEBUGGING; DO NOT REMOVE
    # print("\n-----------------------------------------------------\n",
    #       heading, "\n")
    # for row in rows:
    #     print(row)
    #
    return rows


def day_avoid_ingredient(delivery_date):
    """Get day's avoid ingredients clashes.

    For each client that has ordered a meal for 'delivery_date', find all
    the ingredients that he avoids and identify which of those
    ingredients are included in the main dish for 'delivery_date'.
    Note that the 'avoid ingredients' are the ones that the client specified
    explicitly.

    Args:
        delivery_date: A datetime.date object, the date on which the meals will
            be delivered to the clients.

    Returns:
        A list of named tuples, each one containing the columns specified
        in the SELECT clause.
    """
    query = """
    SELECT member_client.id AS cid,
      member_member.firstname, member_member.lastname,
      order_order.id AS oid,
      order_order.delivery_date,
      meal_menu.id AS menuid,
      meal_menu_component.id AS menucompid,
      meal_ingredient.name AS ingredient,
      meal_component.name AS compname,
      meal_component.component_group,
      order_order_item.id AS oiid,
      order_order_item.order_id AS oiorderid
    FROM member_member
      JOIN member_client ON member_client.member_id = member_member.id
      JOIN order_order ON order_order.client_id = member_client.id
      JOIN meal_menu ON meal_menu.date =                      %(delivery_date)s
      JOIN member_client_avoid_ingredient ON
        member_client_avoid_ingredient.client_id = member_client.id
      JOIN meal_ingredient ON
        meal_ingredient.id = member_client_avoid_ingredient.ingredient_id
      LEFT OUTER JOIN meal_component_ingredient ON
        meal_component_ingredient.ingredient_id = meal_ingredient.id AND
          meal_component_ingredient.date =                    %(delivery_date)s
      LEFT OUTER JOIN meal_component ON
        meal_component.id = meal_component_ingredient.component_id
      LEFT OUTER JOIN order_order_item ON
        order_order_item.order_id = order_order.id AND
          (order_order_item.component_group = meal_component.component_group OR
            (order_order_item.component_group = %(comp_grp_main_dish)s AND
             meal_component.component_group = %(comp_grp_sides)s))
      LEFT OUTER JOIN meal_menu_component ON
        meal_menu_component.component_id = meal_component.id AND
          meal_menu_component.menu_id = meal_menu.id
    WHERE order_order.delivery_date =                     %(delivery_date)s AND
      order_order.status !=                                 %(order_cancelled)s
    ORDER BY member_client.id
    """
    values = {
        "delivery_date": delivery_date,
        "comp_grp_sides": COMPONENT_GROUP_CHOICES_SIDES,
        "comp_grp_main_dish": COMPONENT_GROUP_CHOICES_MAIN_DISH,
        "order_cancelled": ORDER_STATUS_CANCELLED,
    }
    return sql_exec(query, values, "****** Avoid ingredients ******")


def day_restrictions(delivery_date):
    """Get day's restrictions.

    For each client that has ordered a meal for 'delivery_date', find all
    the ingredients that correspond to the restricted items that he specified
    and identify which of those ingredients are included in the main dish
    for 'delivery_date'.

    Args:
        delivery_date: A datetime.date object, the date on which the meals will
            be delivered to the clients.

    Returns:
        A list of named tuples, each one containing the columns specified
        in the SELECT clause.
    """
    query = """
    SELECT member_client.id AS cid,
      member_member.firstname, member_member.lastname,
      order_order.id AS oid,
      order_order.delivery_date,
      meal_menu.id AS menuid,
      meal_menu_component.id AS menucompid,
      meal_restricted_item.name AS restricted_item,
      meal_ingredient.name AS ingredient,
      meal_component.name AS compname,
      meal_component.component_group,
      order_order_item.id AS oiid,
      order_order_item.order_id AS oiorderid
    FROM member_member
      JOIN member_client ON member_client.member_id = member_member.id
      JOIN order_order ON order_order.client_id = member_client.id
      JOIN meal_menu ON meal_menu.date =                      %(delivery_date)s
      JOIN member_restriction ON
        member_restriction.client_id = member_client.id
      JOIN meal_restricted_item ON
        meal_restricted_item.id = member_restriction.restricted_item_id
      LEFT OUTER JOIN meal_incompatibility ON
        meal_incompatibility.restricted_item_id =
          member_restriction.restricted_item_id
      LEFT OUTER JOIN meal_ingredient ON
        meal_incompatibility.ingredient_id = meal_ingredient.id
      LEFT OUTER JOIN meal_component_ingredient ON
        meal_component_ingredient.ingredient_id = meal_ingredient.id AND
          meal_component_ingredient.date =                    %(delivery_date)s
      LEFT OUTER JOIN meal_component ON
        meal_component_ingredient.component_id = meal_component.id
      LEFT OUTER JOIN order_order_item ON
        order_order_item.order_id = order_order.id AND
          (order_order_item.component_group = meal_component.component_group OR
            (order_order_item.component_group = %(comp_grp_main_dish)s AND
             meal_component.component_group = %(comp_grp_sides)s))
      LEFT OUTER JOIN meal_menu_component ON
        meal_menu_component.component_id = meal_component.id AND
          meal_menu_component.menu_id = meal_menu.id
    WHERE order_order.delivery_date =                     %(delivery_date)s AND
      order_order.status !=                                 %(order_cancelled)s
    ORDER BY member_client.id
    """
    values = {
        "delivery_date": delivery_date,
        "comp_grp_sides": COMPONENT_GROUP_CHOICES_SIDES,
        "comp_grp_main_dish": COMPONENT_GROUP_CHOICES_MAIN_DISH,
        "order_cancelled": ORDER_STATUS_CANCELLED,
    }
    return sql_exec(query, values, "****** Restrictions ******")


def day_preparations(delivery_date):
    """Get day's preparations.

    For each client that has ordered a meal for 'delivery_date', find all
    the food preparations that he specified.

    Args:
        delivery_date: A datetime.date object, the date on which the meals will
            be delivered to the clients.

    Returns:
        A list of named tuples, each one containing the columns specified
        in the SELECT clause.
    """
    query = """
    SELECT member_client.id AS cid,
      member_member.firstname, member_member.lastname,
      member_option.name AS food_prep,
      order_order.id AS oid,
      order_order.delivery_date
    FROM member_member
      JOIN member_client ON member_client.member_id = member_member.id
      JOIN member_client_option ON
        member_client_option.client_id = member_client.id
      JOIN member_option ON member_option.id = member_client_option.option_id
      JOIN order_order ON order_order.client_id = member_client.id
      JOIN order_order_item ON
        order_order_item.component_group =           %(comp_grp_main_dish)s AND
          order_order_item.order_id = order_order.id
    WHERE order_order.delivery_date =                     %(delivery_date)s AND
      order_order.status !=                             %(order_cancelled)s AND
      member_option.option_group =                             %(option_group)s
    ORDER BY member_member.lastname, member_member.firstname
    """
    values = {
        "delivery_date": delivery_date,
        "option_group": OPTION_GROUP_CHOICES_PREPARATION,
        "comp_grp_main_dish": COMPONENT_GROUP_CHOICES_MAIN_DISH,
        "order_cancelled": ORDER_STATUS_CANCELLED,
    }
    return sql_exec(query, values, "****** Preparations ******")


def day_delivery_items(delivery_date):
    """Get day's Delivery Items.

    For each client that has ordered something for 'delivery_date', find his
    route and all the details and quantities for the items that have to
    be delivered.

    Args:
        delivery_date: A datetime.date object, the date on which the meals will
            be delivered to the clients.

    Returns:
        A list of named tuples, each one containing the columns specified
        in the SELECT clause.
    """
    query = """
    SELECT member_client.id AS cid,
      member_member.firstname, member_member.lastname,
      member_route.name AS routename,
      order_order_item.total_quantity, order_order_item.size,
      meal_menu.id AS menuid,
      meal_menu_component.id AS mencomid,
      meal_component.id AS component_id, meal_component.component_group,
      meal_component.name AS component_name
    FROM member_member
      JOIN member_client ON member_client.member_id = member_member.id
      JOIN member_route ON member_route.id = member_client.route_id
      JOIN meal_menu ON meal_menu.date = %(delivery_date)s
      JOIN order_order ON order_order.client_id = member_client.id
      JOIN order_order_item ON order_order_item.order_id = order_order.id
      JOIN meal_menu_component ON meal_menu_component.menu_id = meal_menu.id
      JOIN meal_component ON
        meal_component.id = meal_menu_component.component_id AND
        meal_component.component_group = order_order_item.component_group
    WHERE order_order.delivery_date = %(delivery_date)s AND
      order_order.status != %(order_cancelled)s
    ORDER BY member_member.lastname, member_member.firstname
    """
    values = {
        "delivery_date": delivery_date,
        "order_cancelled": ORDER_STATUS_CANCELLED,
    }
    return sql_exec(query, values, "****** Delivery List ******")


KitchenItem = collections.namedtuple(  # Meal specifics for an order.
    "KitchenItem",
    [
        "lastname",  # Client's lastname
        "firstname",  # Client's firstname
        "routename",  # Name of Client's route (ex. 'Mile-end')
        "meal_qty",  # Quantity of main dish servings
        "meal_size",  # Size of main dish
        "incompatible_ingredients",  # Ingredients in main dish that clash
        "sides_clashes",  # Specified restrictions clashes in sides
        "avoid_ingredients",  # All ingredients to avoid for the client
        "restricted_items",  # All restricted items for the client
        "preparation",  # All food preparations for the client
        "meal_components",
    ],
)  # List of MealComponents objects


MealComponent = collections.namedtuple(  # Component specifics for a meal.
    "MealComponent",
    ["id", "name", "qty"],  # Component id  # Component name (ex. 'Rice pudding')
)  # Quantity of this component in the order


def check_for_new_client(kitchen_list, row):
    """Add KitchenItem entry when client is found the first time.

    Modifies kitchen_list by adding an entry.

    Args:
        kitchen_list: A dictionary where the key is an Integer 'client id'
            and the value is a KitchenItem named tuple.
        row: An object having an attribute 'cid' that represents
            an Integer 'client id'.
    """
    if not kitchen_list.get(row.cid):
        # found new client
        kitchen_list[row.cid] = KitchenItem(
            lastname=row.lastname,
            firstname=row.firstname,
            routename=None,
            meal_qty=0,
            meal_size="",
            incompatible_ingredients=[],
            sides_clashes=[],
            avoid_ingredients=[],
            restricted_items=[],
            preparation=[],
            meal_components={},
        )


# End Order.kitchen items helpers


# Order.get_delivery_list helper functions.


DeliveryClient = collections.namedtuple(  # Delivery details for client order.
    "DeliveryClient",
    [
        "firstname",
        "lastname",
        "number",
        "street",
        "apartment",
        "phone",
        "delivery_note",
        "delivery_items",  # list of DeliveryItem objects
        "order_id",
        "include_a_bill",
    ],
)


DeliveryItem = collections.namedtuple(  # Item contained in a delivery.
    "DeliveryItem",
    [
        "component_group",  # String if item is a meal component, else None
        "component_group_trans",  # String or None
        "total_quantity",  # Quantity of item to deliver
        "order_item_type",  # Type of item to deliver (ex. dish, bill, grocery)
        "remark",
        "size",
    ],
)  # Size of item if applicable


def component_group_sorting(component):
    """Sorting sequence for object according to their component_group.

    The sequence is ordered such that the 'main dish' object is first,
    then all the objects having a component_group such as 'dessert',
    'green salad' etc. in alphabetical order, and finally objects
    having an empty component_group.

    Args:
        component: A object having a 'component_group' attribute.

    Returns:
        A string, that can be used as a key to a sort function.
    """
    if component.component_group == COMPONENT_GROUP_CHOICES_MAIN_DISH:
        return "1"
    elif component.component_group != "":
        return "2" + component.component_group
    else:
        return "3"


# End Order.get_delivery_list helpers


class OrderFilter(FilterSet):
    name = CharFilter(method="filter_search", label=_("Search by name"))

    status = ChoiceFilter(choices=(("", ""),) + ORDER_STATUS)

    class Meta:
        model = Order
        fields = ["status", "delivery_date"]

    def filter_search(self, queryset, field_name, value):
        if not value:
            return queryset

        names = value.split(" ")

        name_contains = Q()

        for name in names:
            firstname_contains = Q(client__member__firstname__icontains=name)

            name_contains |= firstname_contains

            lastname_contains = Q(client__member__lastname__icontains=name)
            name_contains |= lastname_contains

        return queryset.filter(name_contains)


class DeliveredOrdersByMonth(FilterSet):
    delivery_date = CharFilter(method="filter_period")

    class Meta:
        model = Order
        fields = "__all__"

    def filter_period(self, queryset, field_name, value):
        if not value:
            return None

        year, month = value.split("-")
        return queryset.filter(
            status="D",
            delivery_date__year=year,
            delivery_date__month=month,
        )


class Order_item(models.Model):
    class Meta:
        verbose_name_plural = _("order items")

    order = models.ForeignKey(
        "order.Order",
        verbose_name=_("order"),
        related_name="orders",
        on_delete=models.CASCADE,
    )

    price = models.DecimalField(max_digits=6, decimal_places=2, verbose_name=_("price"))

    billable_flag = models.BooleanField(
        verbose_name=_("billable flag"),
    )

    size = models.CharField(
        verbose_name=_("size"),
        max_length=1,
        null=True,
        blank=True,
        choices=SIZE_CHOICES,
    )

    order_item_type = models.CharField(
        verbose_name=_("order item type"),
        max_length=20,
        choices=ORDER_ITEM_TYPE_CHOICES,
    )

    remark = models.CharField(
        max_length=256,
        verbose_name=_("remark"),
        null=True,
        blank=True,
    )

    total_quantity = models.IntegerField(
        verbose_name=_("total quantity"),
    )

    free_quantity = models.IntegerField(
        verbose_name=_("free quantity"),
        null=True,
        blank=True,
    )

    component_group = models.CharField(
        max_length=100,
        choices=COMPONENT_GROUP_CHOICES,
        verbose_name=_("component group"),
        null=True,
        blank=True,
    )

    def __str__(self):
        return (
            f"<For delivery on:> {str(self.order.delivery_date)} <order_item_type:>"
            f" {self.order_item_type} <component_group:> {self.component_group}"
        )

    def get_billable_flag_display(self):
        if self.billable_flag:
            return _("Yes")
        else:
            return _("No")

    def get_order_item_type_display(self):
        if self.order_item_type == "meal_component":
            return _("Meal component")
        elif self.order_item_type == "delivery":
            return _("Delivery")
        elif self.order_item_type == "pickup":
            return _("Pickup")
        elif self.order_item_type == "visit":
            return _("Visit")
        else:
            return dict(ORDER_ITEM_TYPE_CHOICES)[self.order_item_type]

    @property
    def is_a_client_bill(self):
        """
        An order item may indicate a bill to the client for this order.
        """
        return self.order_item_type == "delivery" and self.component_group is None


class OrderStatusChange(models.Model):
    class Meta:
        ordering = ["change_time"]

    order = models.ForeignKey(
        Order, on_delete=models.CASCADE, related_name="status_changes"
    )

    status_from = models.CharField(max_length=1, choices=ORDER_STATUS)

    status_to = models.CharField(max_length=1, choices=ORDER_STATUS)

    reason = models.CharField(max_length=200, blank=True, default="")

    change_time = models.DateTimeField(auto_now_add=True)

    def __str__(self):
        return (
            f"Order #{self.order.pk} status update: from "
            f"{self.get_status_from_display()} to {self.get_status_to_display()}, on "
            f"{self.change_time}"
        )

    def clean(self):
        """
        Make sure this is valid in regards to Order.
        """
        if self.order.status != self.status_from:
            raise ValidationError(
                _("Invalid order status update."), code="status_from_incorrect"
            )

    def save(self, *a, **k):
        """Process a scheduled change when saving."""
        self.full_clean()  # we defined clean method so we need to override
        super().save(*a, **k)
        self.order.status = self.status_to
        self.order.save()



# Generated by Django 1.9.7 on 2016-08-03 22:17

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("order", "0005_fix241b"),
    ]

    operations = [
        migrations.AlterField(
            model_name="order",
            name="status",
            field=models.CharField(
                choices=[
                    ("O", "Ordered"),
                    ("D", "Delivered"),
                    ("N", "No Charge"),
                    ("B", "Billed"),
                    ("P", "Paid"),
                ],
                max_length=1,
                verbose_name="order status",
            ),
        ),
    ]



# Generated by Django 1.9.5 on 2016-07-14 12:06

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("meal", "0002_ingredient_ingredient_group"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="ingredient",
            options={"verbose_name_plural": "ingredients"},
        ),
        migrations.AlterField(
            model_name="component",
            name="component_group",
            field=models.CharField(
                choices=[
                    ("main dish", "Main dish"),
                    ("dessert", "Dessert"),
                    ("diabetic dessert", "Diabetic dessert"),
                    ("fruit salad", "Fruit salad"),
                    ("green salad", "Green salad"),
                    ("pudding", "Pudding"),
                    ("compote", "Compote"),
                ],
                max_length=100,
                verbose_name="component group",
            ),
        ),
    ]



import csv
from datetime import date

from django.core.management import call_command
from django.core.management.base import BaseCommand

from souschef.member.models import (
    Client,
    Member,
    Route,
)


class Command(BaseCommand):
    help = "Data: import clients from given csv file."

    ROW_MID = 0
    ROW_FIRSTNAME = 1
    ROW_LASTNAME = 2
    ROW_BIRTHDATE = 3
    ROW_STATUS = 4
    ROW_CREATED = 5
    ROW_GENDER = 6
    ROW_LANG = 7
    ROW_PAYSCALE = 8
    ROW_ALERT = 9
    ROW_DELIVERY_NOTES = 10
    ROW_DELIVERY_TYPE = 11
    ROW_ROUTE = 12

    def add_arguments(self, parser):
        parser.add_argument(
            "--file",
            default=False,
            help="Import mock data instead of actual data",
        )

    def handle(self, *args, **options):
        # Load fixtures for the routes
        fixture_filename = "routes.json"
        call_command("loaddata", fixture_filename)

        file = "mock_clients.csv" if options["file"] else "clients.csv"

        with open(file) as f:
            reader = csv.reader(f, delimiter=";")
            for row in reader:
                row_created = (
                    row[self.ROW_CREATED]
                    if row[self.ROW_CREATED] != ""
                    else date.today()
                )

                member, created = Member.objects.update_or_create(
                    mid=row[self.ROW_MID],
                    defaults={
                        "firstname": row[self.ROW_FIRSTNAME],
                        "lastname": row[self.ROW_LASTNAME],
                        "updated_at": row_created,
                    },
                )

                route, created = Route.objects.get_or_create(name=row[self.ROW_ROUTE])
                if created:
                    err_msg = "A new route has been created."
                    self.stdout.write(self.style.WARNING(err_msg))
                    route = None

                client, created = Client.objects.update_or_create(
                    member=member,
                    defaults={
                        "billing_member": member,
                        "birthdate": row[self.ROW_BIRTHDATE],
                        "status": row[self.ROW_STATUS],
                        "gender": row[self.ROW_GENDER],
                        "alert": row[self.ROW_ALERT],
                        "delivery_type": row[self.ROW_DELIVERY_TYPE],
                        "delivery_note": row[self.ROW_DELIVERY_NOTES],
                        "route": route,
                        "language": row[self.ROW_LANG],
                        "rate_type": row[self.ROW_PAYSCALE],
                    },
                )

                self.stdout.write(
                    self.style.SUCCESS(
                        "{} successfully {}.".format(
                            client, "created" if created else "updated"
                        )
                    )
                )

                # Override creation date
                client.member.created_at = row_created

                client.member.save()

                # Add Client option meals_schedule
                # client.set_simple_meals_schedule([])



# Generated by Django 1.10 on 2016-11-22 04:58

from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("order", "0011_auto_20161014_1901"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="order",
            options={"ordering": ["-delivery_date"], "verbose_name_plural": "orders"},
        ),
    ]



from django.contrib import admin

from souschef.note.models import (
    Note,
    NoteCategory,
    NotePriority,
)


class NoteAdmin(admin.ModelAdmin):
    list_display = (
        "date_created",
        "date_modified",
        "author",
        "is_read",
        "is_deleted",
        "priority",
        "category",
        "client",
        "note",
    )
    list_filter = (
        "priority",
        "category",
        "is_read",
        "is_deleted",
    )


# Register your models here.
admin.site.register(Note, NoteAdmin)
admin.site.register((NotePriority, NoteCategory))



# Generated by Django 1.10 on 2016-08-26 00:07

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("meal", "0006_auto_20160826_0007"),
        ("member", "0013_auto_20160820_2322"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="clientscheduledstatus",
            options={"ordering": ["change_date"]},
        ),
        migrations.AddField(
            model_name="client",
            name="components_to_avoid",
            field=models.ManyToManyField(
                through="member.Client_avoid_component", to="meal.Component"
            ),
        ),
        migrations.AddField(
            model_name="client",
            name="ingredients_to_avoid",
            field=models.ManyToManyField(
                through="member.Client_avoid_ingredient", to="meal.Ingredient"
            ),
        ),
        migrations.AddField(
            model_name="client",
            name="options",
            field=models.ManyToManyField(
                through="member.Client_option", to="member.Option"
            ),
        ),
        migrations.AddField(
            model_name="client",
            name="restrictions",
            field=models.ManyToManyField(
                through="member.Restriction", to="meal.Restricted_item"
            ),
        ),
        migrations.AlterField(
            model_name="clientscheduledstatus",
            name="client",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                related_name="scheduled_statuses",
                to="member.Client",
            ),
        ),
    ]
